/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import java.io.*;
import javax.swing.*;

/**
  * This class is also another top level program which will coordinate to:
  *    1) copy Web Server Access Log file to a text file in the input directory
  *    2) store data either in database warehouse or in text files warehouse
  *    3) delete copied access log file from the input directory
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class DataCollector {
   private int count;              // 25 will store data in database warehouse
                                   // 26 will store data in text file warehouse
   private JTextArea out;          // to display at top most GUI
   private static String[] str;    // argument for copy command
   private Config config;          // to get copy command, path, source file and target file

   /**
     * The constructor will initiate variables
     */

   public DataCollector( int c, JTextArea o ) {
      count = c;
      out = o;

      config = Config.Factory();
      // getting copy command
      String cp = config.getCopyCommand();
      // getting file path
      String path = config.getPath();
      // getting web server access log file name
      String f1 = config.getServerAccessLogFile();
      // getting log access file name which will be used as input file for further processing
      String f2 = config.getAccessLogFile();

      // building argument for copy command
      str = new String [3];
      str[0] = cp;
      str[1] = f1;
      str[2] = path + f2;
   }

   /**
     * Method operation will cordinate copying the Access Log file,
     * storing data in either database  warehouse or in text files warehouse,
     * and deleting the copied file
     */

   public void operation() {
      copyAccessLogFile();
      if (count == 25) {
         sendToDatabase();
      } else if (count == 26) {
         sendToTextFiles();
      }
      deleteCopiedLogFile();
   }

   /**
     * Method copyAccessLogFile will create an object of CopyLog class
     * and call it's copyLogFile method
     * to copy Access Log file to a text file in the input directory
     */

   public void copyAccessLogFile() {
      CopyLog cl = new CopyLog( str );
      cl.copyLogFile();
   }

   /**
     * Method sendToDatabase will create an object of
     * DBDataCollector class and call it's processData method
     * to store data in database warehouse
     */

   public void  sendToDatabase() {
      DBDataCollector dbdc = new DBDataCollector( out );
      System.out.println( "\nStart of processing access record "
                        + "to be stored in database" );
      dbdc.processData();
      System.out.println( "End of processing access record "
                        + "to be stored in database\n" );
   }

   /**
     * Method sendToTextFiles will create an object of
     * TFDataCollector class and call it's processData method
     * to store data in text files  warehouse in the output directory
     */

   public void  sendToTextFiles() {
      TFDataCollector tfdc = new TFDataCollector( out );
      System.out.println( "\nStart of processing access record "
                        + "to be stored in text files" );
      tfdc.processData();
      System.out.println( "End of processing access record  "
                        + "to be stored in text files\n" );
   }

   /**
     * Method deleteCopiedLogFile will create an object of File class with
     * the previously copied access log file name as argument and
     * check if the file exists by call it's exists method.
     * If the file exists, delete the file to avoid storing a large file which will not be processed any further
     */

   public void deleteCopiedLogFile() {
     File f = new File( str[2] );
     if( f.exists() ) {
        f.delete();
     }
   }
} // DataCollector
