/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import java.io.*;
import java.util.*;
import java.sql.*;

/**
  * This program will implement all related activities
  * for storing records in the text files warehouse as follows:
  *    1) create all files of the text files warehouse in output directory
  *    2) delete all files of the text files warehouse from output directory
  *    3) get time stamp of the most recent stored access record
  *    4) check if the record is in the text files warehouse
  *    5) insert entries into UserDimTbl, ProtocolDimTbl, PathDimTbl, and LogFactTble
  *    6) determine sessionID for each web access
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class DataCollectorControl {
   private String path;

   /**
     * The constructor will get file path name from singleton Config object
     */

   public DataCollectorControl() {
      // the singleton object which will store file path
      Config config = Config.Factory();
      // getting file path
      path = config.getPath();
   }

  /**
    * Method getLastAccessTime get time stamp of the most recent access record in the text files warehouse
    *
    * @exception   FileNotFoundException   if input file can not be located
    *
    * @exception   IOException             io exception
    *
    * @return                              time stamp of the most recent web server log access
    */

   public MyDate getLastAccessTime() {
      String str = "";
      String hold = "";
      MyDate d = null;
      MyDate d1 = null;

      BufferedReader reader = null;
      try {
         reader = new BufferedReader(
                     new FileReader( path + "output/LogFactTbl" ) );
         str = reader.readLine();
         str = reader.readLine();
         // while loop to find time stamp of the most recent web server access
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
            }
            d1 = new MyDate( hold );

            // comparing time stamps to get time stamp of more recent access
            if( d == null || d1.compareTo( d ) >= 0 ) {
               d = d1;
            }
            str = reader.readLine();
         }
         reader.close();

      } catch( FileNotFoundException fnf ) {
         System.out.println("Control: Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return d;
   }

   /**
     * Method isInStore checks if the record is in the warehouse
     *
     * @param       arg                     arg is a array of Strings
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException             io exception
     *
     * @return                              true if the record is in store, false otherwise
     */

   public boolean isInStore( String[] arg ) {
      // IDs for the record stored in arg
      String timeID = "";
      int userID = -1;
      int protocolID = -1;
      int pathID = -1;

      // IDs for the currently accessed record from the warehouse
      String tID = "";
      int uID = -1;
      int protID = -1;
      int paID = -1;

      boolean flag = false;
      String str = "";
      String hold = "";

      BufferedReader reader = null;
      try {
	 String ipAddress = "";
	 String hostName = "";
	 String userName = "";

         reader = new BufferedReader(
                    new FileReader( path + "output/UserDimTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to find userID from UserDimTble for the record
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               uID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               ipAddress = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               hostName = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               userName = tok.nextToken();
            }
            // compare to find a match
            if( ipAddress.equals( arg[0] ) ) {
               userID = uID;
               break;
            }
            str = reader.readLine();
         }
         reader.close();
         // if record has no userID in UserDimTbl,
         // the record is not in the warehouse
         if( userID == -1 ) {
	    return flag;
	 }

	 String protocolName = "";
	 String methodName = "";
	 String status = "";

         reader = new BufferedReader(
                    new FileReader( path + "output/ProtocolDimTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to find protocolID from ProtocolDimTble for the record
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               protID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               protocolName = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               methodName = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               status = tok.nextToken();
            }
            // compare to find a match
            if( protocolName.equals( arg[9] )
                   &&  methodName.equals( arg[5] )
                        && status.equals( arg[10] ) ) {
               protocolID = protID;
               break;
            }
            str = reader.readLine();
         }
         reader.close();
         // if record has no protocolID in ProtocolDimTbl,
         // the record is not in the warehouse
         if( protocolID == -1 ) {
	    return flag;
	 }

	 String pathName = "";
	 String fileName = "";
	 String fileType = "";

         reader = new BufferedReader(
                     new FileReader( path + "output/PathDimTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to find pathID from PathDimTble for the record
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               paID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               pathName = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               fileName = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               fileType = tok.nextToken();
            }
            // compare to find a match
            if( pathName.equals( arg[6] )
                &&  fileName.equals( arg[7] )
                    && fileType.equals( arg[8] ) ) {

               pathID = paID;
               break;
            }
            str = reader.readLine();
         }
         reader.close();
         // if record has no pathID in PathDimTbl,
         // the record is not in the warehouse
         if( pathID == -1 ) {
	    return flag;
	 }
         // logical timestring
         timeID = arg[3] + "/" + arg[4];
         // initializing the IDs for the current record being accessed from the LogFactTbl
         tID = "";
         uID = -1;
         protID = -1;
         paID = -1;

         reader = new BufferedReader(
                      new FileReader( path + "output/LogFactTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to find the entry in LogFactTbl which match with the access record
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               tID = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               uID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               protID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               paID = Integer.parseInt( hold );
            }
            int sID = -1;
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               sID = Integer.parseInt( hold );
            }
            int nByte = -1;
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               nByte = Integer.parseInt( hold );
            }
            int nbyte;
            if( !arg[11].equals( "-" ) ) {
               nbyte = Integer.parseInt( arg[11] );
            } else {
               nbyte = 0;
            }
            // compare to find a match
            if( tID != null && tID.equals( timeID )
                  && uID != -1 && uID == userID
                    && protID != -1 && protID == protocolID
                      && paID != -1 && paID == pathID
                        && sID != -1
                          && nByte != -1 && nByte == nbyte ) {

               flag = true;
               break;
            }
            str = reader.readLine();
         }
         reader.close();
      } catch( FileNotFoundException fnf ) {
         System.out.println("Control: Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return flag;
   }

   /**
     * Method insertToUserDimTbl will return userID such that
     *    1) if the user information exist in the UserDimTbl,
     *       it will return the existing userID
     *    2) if the user information does not exist in the UserDimTbl,
     *       it will insert the information to the UserDimTbl and
     *       return the new userID
     *
     * @param       ipAddrs                is IP Address of the user
     *
     * @param       hostName               is host name of the IP Address
     *
     * @param       userName               is user name of the IP Address
     *
     * @exception   FileNotFoundException  if input file can not be located
     *
     * @exception   IOException            for io exception
     *
     * @return                             userID for the user
     */

   public int insertToUserDimTbl( String ipAddrs,
                                  String hostName,
                                  String userName ) {
      int uID = 1;
      int maxUID = 0;
      boolean flag = false;

      String ip = "";
      String host = "";
      String user = "";

      String str = "";
      String hold = "";
      BufferedReader reader = null;

      try {
         reader = new BufferedReader(
                            new FileReader( path + "output/UserDimTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to get userID, if the user information exist in UserDimTbl
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               uID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               ip = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               host = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               user = tok.nextToken();
            }
            // compare to find a match
            if( ip != null && ip.equals( ipAddrs ) ) {
               flag = true;
               break;
            }
            str = reader.readLine();
         }
         reader.close();

         // if-else selection to get userID
         if( flag ) {
            // if the user information exist in UserDimTbl return userID
            return uID;
         } else {
            // if the user information does not exist in UserDimTbl,
            // insert user information into UserDimTbl
            uID = 1;
            reader = new BufferedReader(
                         new FileReader( path + "output/UserDimTbl" ) );

            str = reader.readLine();
            str = reader.readLine();
            // finding largest userID used for the user so far
            while( str != null ) {
               StringTokenizer tok = new StringTokenizer( str, " \t" );

               if( tok.hasMoreTokens() ) {
                  hold = tok.nextToken();
                  uID = Integer.parseInt( hold );
                  if( uID > maxUID ) {
                     maxUID = uID;
                  }
               }
               str = reader.readLine();
            }
            reader.close();
            // getting next userID for the user
            maxUID++;
            PrintWriter out = new PrintWriter (
                                  new BufferedWriter (
                                      new FileWriter (
                                        path + "output/UserDimTbl", true ) ) );
            // creating user information string to be stored to UserDimTbl
            String s = "";
            s += maxUID + " ";
            while( s.length() < 15 ) {
               s += " ";
            }
            s += ipAddrs + " ";
            while( s.length() < 35 ) {
               s += " ";
            }
            s += hostName + " ";
            while( s.length() < 100 ) {
               s += " ";
            }
            s += userName;
            // writing user information int UserDimTbl
            out.println( s );
            out.close();
            return maxUID;
         }

      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return -1;
   }

   /**
     * Method insertToProtocolDimTbl will return protocolID such that
     *    1) if the protocol information exist in the ProtocolDimTbl,
     *       it will return the existing protocolID
     *    2) if the protocol information does not exist in the ProtocolDimTbl,
     *       it will insert the information to the ProtocolDimTbl and
     *       return the new protocolID
     *
     * @param       prot                   prot is HTTP protocol
     *
     * @param       mt                     mt is method
     *
     * @param       st                     st is status
     *
     * @exception   FileNotFoundException  if input file can not be located
     *
     * @exception   IOException            for io exception
     *
     * @return                             protocolID for the protocol
     */

   public int insertToProtocolDimTbl( String prot, String mt, String st ) {
      int protID = 1;
      int maxProtID = 0;
      boolean flag = false;

      String p = "";
      String m = "";
      String s = "";

      String str = "";
      String hold = "";
      BufferedReader reader = null;

      try {
         reader = new BufferedReader(
                           new FileReader( path + "output/ProtocolDimTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to get protocolID, if the protocol information exist in ProtocolDimTbl
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               protID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               p = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               m = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               s = tok.nextToken();
            }
            // compare to find a match
            if( p != null && p.equals( prot )
                  && m != null && m.equals( mt )
                       && s != null && s.equals( st ) ) {
               flag = true;
               break;
            }
            str = reader.readLine();
         }
         reader.close();

         // if-else selection to get protocolID
         if( flag ) {
             // if the protocol information exist in ProtocolDimTbl
            return protID;
         } else {
            // if the protocol information does not exist in ProtocolDimTbl,
            // insert protocol information into ProtocolDimTbl
            protID = 1;
            reader = new BufferedReader(
                         new FileReader( path + "output/ProtocolDimTbl" ) );
            str = reader.readLine();
            str = reader.readLine();
            // finding largest protocolID used for the protocol so far
            while( str != null ) {
               StringTokenizer tok = new StringTokenizer( str, " \t" );

               if( tok.hasMoreTokens() ) {
                  hold = tok.nextToken();
                  protID = Integer.parseInt( hold );
                  if( protID > maxProtID ) {
                     maxProtID = protID;
                  }
               }
               str = reader.readLine();
            }
            reader.close();
            // getting next protocolID for the protocol
            maxProtID++;
            PrintWriter out = new PrintWriter (
                               new BufferedWriter (
                                 new FileWriter (
                                    path + "output/ProtocolDimTbl", true ) ) );
            // creating protocol information string to be stored to ProtocolDimTbl
            s = "";
            s += maxProtID + " ";
            while( s.length() < 15 ) {
               s += " ";
            }
            s += prot + " ";
            while( s.length() < 35 ) {
               s += " ";
            }
            s += mt + " ";
            while( s.length() < 50 ) {
               s += " ";
            }
            s += st;
            // writing to protocolDimTbl
            out.println( s );
            out.close();
            return maxProtID;
         }
      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return -1;
   }

   /**
     * Method insertToPathDimTbl will return pathID such that
     *    1) if the path information exist in the PathDimTbl,
     *       it will return the existing pathID
     *    2) if the path information does not exist in the PathDimTbl,
     *       it will insert the information to the PathDimTbl and
     *       return the new pathID
     *
     * @param       pa                     pa contains path name
     *
     * @param       fi                     fi contains file name
     *
     * @param       ty                     ty contains file type name
     *
     * @exception   FileNotFoundException  if input file can not be located
     *
     * @exception   IOException            for io exception
     *
     * @return                             pathID for the path
     */

   public int insertToPathDimTbl( String pa, String fi, String ty ) {
      int paID = 1;
      int maxPaID = 0;
      boolean flag = false;

      String p = "";
      String f = "";
      String t = "";

      String str = "";
      String hold = "";
      BufferedReader reader = null;

      try {
         reader = new BufferedReader(
                      new FileReader( path + "output/PathDimTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to get pathID, if the path information exist in PathDimTbl
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               paID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               p = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               f = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               t = tok.nextToken();
            }
            // to find a match
            if( p != null && p.equals( pa )
                  && f != null && f.equals( fi )
                       && t != null && t.equals( ty ) ) {
               flag = true;
               break;
            }
            str = reader.readLine();
         }
         reader.close();

         // if selection to get pathID
         if( flag ) {
            // if the path information exist in PathDimTbl
            return paID;
         } else {
            // if the path information does not exist in PathDimTbl
            // insert path information into PathDimTbl
            paID = 1;
            reader = new BufferedReader(
                         new FileReader( path + "output/PathDimTbl" ) );

            str = reader.readLine();
            str = reader.readLine();
            // finding largest pathID for the path
            while( str != null ) {
               StringTokenizer tok = new StringTokenizer( str, " \t" );

               if( tok.hasMoreTokens() ) {
                  hold = tok.nextToken();
                  paID = Integer.parseInt( hold );
                  if( paID > maxPaID ) {
                     maxPaID = paID;
                  }
               }
               str = reader.readLine();
            }
            reader.close();
            // getting next pathID for the path
            maxPaID++;
            PrintWriter out = new PrintWriter (
                                  new BufferedWriter (
                                      new FileWriter (
                                        path + "output/PathDimTbl", true ) ) );
            // creating path information to be inserted to PathDimTbl
            String s = "";
            s += maxPaID + " ";
            while( s.length() < 15 ) {
               s += " ";
            }
            s += pa + " ";
            while( s.length() < 75 ) {
               s += " ";
            }
            s += fi + " ";
            while( s.length() < 120 ) {
               s += " ";
            }
            s += ty;
            // inserting path information into PathDimTbl
            out.println( s );
            out.close();
            return maxPaID;
         }

      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return -1;
   }

   /**
     * Method insertToLogFactTbl will insert an access record into
     * LogFactTbl along with sessionID.
     *
     * @param      timeID                 logical ID for time stamp
     *
     * @param      userID                 foreign key refers to corresponding
     *                                    primary key in UserDimtbl
     *
     * @param      protocolID             foreign key refers to corresponding
     *                                    primary key in ProtocolDimtbl
     *
     * @param      pathID                 foreign key refers to corresponding
     *                                    primary key in PathDimtbl
     *
     * @param      b                      number of bytes accessed
     *
     * @exception  FileNotFoundException  if input file can not be located
     *
     * @exception  IOException            for io exception
     *
     * @return                            int to indicate success of insertion
     */

   public int insertToLogFactTbl( String timeID, int userID, int protocolID,
                                  int pathID, int b ) {
      int count = -1;
      int sessionID = 1;
      PrintWriter out = null;

      // method call to get sessionID
      sessionID = getSessionID( timeID, userID );
      try {
         out = new PrintWriter (
                   new BufferedWriter (
                       new FileWriter ( path + "output/LogFactTbl", true ) ) );
         // creating record information to be inserted into LogFactTbl
         String s = "";
         s += timeID + " ";
         while( s.length() < 30 ) {
            s += " ";
         }
         s += userID + " ";
         while( s.length() < 40 ) {
            s += " ";
         }
         s += protocolID + " ";
         while( s.length() < 55 ) {
            s += " ";
         }
         s += pathID + " ";
         while( s.length() < 65 ) {
            s += " ";
         }
         s += sessionID + " ";
         while( s.length() < 80 ) {
            s += " ";
         }
         s += b;
         // inserting the record into LogFactTbl
         out.println( s );
         out.close();
         count = 1;

      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return count;
   }

   /**
     * Method getSessionID will find sessionID for an access record
     *
     * @param       timeID                 time stamp of an access record
     *
     * @param       userID                 userID to identify user
     *
     * @exception   FileNotFoundException  if input file can not be located
     *
     * @exception   IOException            for io exception
     *
     * @return                             sessionID as int
     */

   public int getSessionID( String timeID, int userID ) {
      BufferedReader reader = null;
      String str = "";
      String hold = "";
      StringTokenizer tok;

      String tID = "";
      int uID = -1;
      int sessionID = 1;
      int sID = 1;
      int maxSID = 1;

      MyDate d = new MyDate( timeID );
      MyDate d1 = null;
      MyDate d2 = null;

      try {
         reader = new BufferedReader(
                    new FileReader( path + "output/LogFactTbl" ) );
         str = reader.readLine();
         str = reader.readLine();
         // getting next sessionID to be used
         while( str != null ) {
            tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               tID = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               uID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               sID = Integer.parseInt( hold );

               if( sID >= maxSID ) {
                  maxSID = sID + 1;
               }
            }
            // getting most recent sessionID for the user
            if( uID == userID ) {
               d2 = new MyDate( tID );
               if( d1 == null || d2.compareTo( d1 ) >= 0 ) {
                  d1 = d2;
                  sessionID = sID;
               }
            }
            str = reader.readLine();
         }
         reader.close();
      } catch( FileNotFoundException fnf ) {
         System.out.println( "Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ) {
         System.out.println( ioe.getMessage() );
      }

      // if most recent access of the user is not in the same session
      // with the current access, sessionID will be next sessionID to be used
      if( d == null || d1 == null || d.notInSameSession( d1 ) ) {
         sessionID = maxSID;
      }
      return  sessionID;
   }
} // Control
