/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import javax.swing.*;
import java.io.*;
import java.util.*;

/**
  * This program will implement all related activities for creating
  * data set in text file from the text files warehouse.
  * It will create the following data sets:
  *    1) time based data set
  *    2) user based data set
  *    3) protocol based data set
  *    4) path based data set
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class DataSetControl {
   private JTextArea out;             // for display

   private int total;                // data set size
   private String fileName;          // data set file name
   private String fileHeader;        // store header string for the data sets

   private String startTime;          // data set start time
   private String endTime;            // data set end time

   private String path;               // file path


   /**
    * The constructor will initialize variables
    */

   public DataSetControl( String fName, String st, String et, int t, JTextArea o ) {
      out = o;

      total = t;
      fileName = fName;

      startTime = st;
      endTime = et;
      // the singleton object which will store file path
      Config config = Config.Factory();
      // getting file path
      path = config.getPath();

      // creating header of the dataset
      String s = "";
      s += "timeID ";
      while( s.length() < 30 ) {
         s += " ";
      }
      s += "userID ";
      while( s.length() < 40 ) {
         s += " ";
      }
      s += "protocolID ";
      while( s.length() < 55 ) {
         s += " ";
      }
      s += "pathID ";
      while( s.length() < 65 ) {
         s += " ";
      }
      s += "sessionID ";
      while( s.length() < 80 ) {
         s += " ";
      }
      s += "nbyte";
      // getting file header
      fileHeader = s;
   }

   /**
     * Method createTimeBasedDS will check time stamp of all the enrtries of the LogFactTbl
     * and compare with the staring time and ending time of the required data set.
     * If time stamp of the entry is equal to or more recent than the staring time
     * and equal to or earlier than the ending time, the entry is included into the data set
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException              for io exception
     */

   public void createTimeBasedDS() {
      int index = 0;                // current number of entry in the data set
      String str = "";              // string to be added into data set
      String tID = "";              // timeID of the current entry at warehouse
      MyDate d = null;              // current time
      MyDate d1 = null;             // start time
      MyDate d2 = null;             // end time
      ArrayList al;                 // to create visit sequence
      PrintWriter pw = null;
      BufferedReader reader = null;

      d1 = new MyDate( startTime );
      d2 = new MyDate( endTime );
      al = new ArrayList();

      try {
         pw = new PrintWriter (
                   new BufferedWriter (
                       new FileWriter ( path + "dataset/" + fileName ) ) );

         pw.println( fileHeader );

         reader = new BufferedReader(
                    new FileReader( path + "output/LogFactTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to compare and add into data set
         while( str != null && index < total ) {
            String temp = str;
            StringTokenizer tok = new StringTokenizer( temp, " \t" );
            if( tok.hasMoreTokens() ) {
               tID = tok.nextToken();
            }
            // time of the current entry
            d = new MyDate( tID );
            // compare
            if( ( d.compareTo( d1 ) >= 0 ) && ( d.compareTo( d2 ) <= 0 ) ) {
               // adding to the data set
               pw.println( str );
               index++;
               // to create visit sequence
               MyRecord mr = new MyRecord( str );
               al.add( ( Object ) mr );
            }
            if( index >= 1 && index % 100 == 0 ) {
               System.out.println( "Till now size of the Data set: " + index + " ..." );
            }
            str = reader.readLine();
         }
         reader.close();
         // summery string
         String f = "";
         f = "\n\n*****Time based Dataset from Text File Warehouse*****"
           + "\nStart Time: " + startTime
           + "\nEnd Time: " + endTime
           + "\nRequested size of the Data set: " + total
           + "\nActual size of the Data set: " + index;
         // writing summery
         pw.println( f );
         pw.close();
         // for a seperate file showing sequence of web page visited
         DataSetSequence dss = new DataSetSequence( al, fileName, f );
         dss.buildWebpageVisitSeq();
         // for display
         out.setText( f );
      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method createUserBasedDS will find userID for the user from UserDimTbl.
     * Then it will find all the entries for the userID from the LogFactTbl and
     * compare the time stamp of the entries with the staring time and ending time of the required data set.
     * If time stamp of the entry is equal to or more recent than the staring time
     * and equal to or earlier than the ending time, the entry is included into the data set

     *
     * @param       user                     ip address of the user
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException              for sql exception
     */

   public void createUserBasedDS( String user ) {
      int index = 0;               // current number of entry in the data set
      int userID = -1;             // userID for the user
      int uID = -1;                // userID in int for the current entry
      String u = "";               // userID in String for the current entry
      String str = "";             // string to be added into data set
      String tID = "";             // timeID of the current entry at warehouse
      MyDate d = null;              // current time
      MyDate d1 = null;             // start time
      MyDate d2 = null;             // end time
      ArrayList al;                 // to create visit sequence
      PrintWriter pw = null;
      BufferedReader reader = null;

      d1 = new MyDate( startTime );
      d2 = new MyDate( endTime );
      al = new ArrayList();

      // method call to get userID for the user
      userID = getUserID( user );
      // if there is no record of the user, return with message
      if( userID == -1 ) {
         out.setText( "\n\nThere is no data entry for the user\n"
                    + user
                    + "\nin the warehouse.\n"
                    + fileName + " file is not created." );
         return;
      }

      try {
         pw = new PrintWriter (
                   new BufferedWriter (
                       new FileWriter ( path + "dataset/" + fileName ) ) );
         //writing header
         pw.println( fileHeader );

         reader = new BufferedReader(
                    new FileReader( path + "output/LogFactTbl" ) );
         str = reader.readLine();
         str = reader.readLine();
         // while loop to compare and add into data set
         while( str != null && index < total ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               tID = tok.nextToken();
            }
            // time of the current entry
            d = new MyDate( tID );

            if( tok.hasMoreTokens() ) {
               u = tok.nextToken();
               uID = Integer.parseInt( u );
            }

            // compare
            if( uID == userID
                   && ( d.compareTo( d1 ) >= 0 ) && ( d.compareTo( d2 ) <= 0 ) ) {
               // adding to the data set
               pw.println( str );
               index++;
               // to create visit sequence
               MyRecord mr = new MyRecord( str );
               al.add( ( Object ) mr );
            }
            if( index >= 1 && index % 100 == 0 ) {
               System.out.println( "Till now size of the Data set: " + index + " ..." );
            }
            str = reader.readLine();
         }
         reader.close();
         //summery string
         String f = "";
         f = "\n\n*****User based Dataset from Text File Warehouse*****"
           + "\nUser: " + user
           + "\nStart Time: " + startTime
           + "\nEnd Time: " + endTime
           + "\nRequested size of the Data set: " + total
           + "\nActual size of the Data set: " + index;
         //writing summery
         pw.println( f );
         pw.close();
         // for a seperate file showing sequence of web page visited
         DataSetSequence dss = new DataSetSequence( al, fileName, f );
         dss.buildWebpageVisitSeq();
         //for display
         out.setText( f );
      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method createProtocolBasedDS will find protocolID for the protocol from ProtocolDimTbl.
     * Then it will find all the entries for the protocolID
     * from the LogFactTbl and compare the time stamp of the entries with
     * the staring time and ending time of the required data set.
     * If time stamp of the entry is equal to or more recent than the staring time
     * and equal to or earlier than the ending time, the entry is included into the data set
     *
     * @param       protocol                 array of string for protocol, method, and status
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException              for sql exception
     */

   public void createProtocolBasedDS( String[] protocol ) {
      int index = 0;         // current number of entry in the data set
      int protocolID = -1;   // protocolID for the protocol, method, and status
      int protID = -1;       // protocolID in int for current entry
      String prot = "";      // protocolID in String for current entry
      String str = "";       // string to be added into data set
      String tID = "";       // timeID of the current entry at warehouse
      MyDate d = null;              // current time
      MyDate d1 = null;             // start time
      MyDate d2 = null;             // end time
      ArrayList al;                 // to create visit sequence
      PrintWriter pw = null;
      BufferedReader reader = null;

      d1 = new MyDate( startTime );
      d2 = new MyDate( endTime );
      al = new ArrayList();

      // method call to find protocolID of the protocol
      protocolID = getProtocolID( protocol );
      // if there is no record of the protocol, return with message
      if( protocolID == -1 ) {
         out.setText( "\n\nThere is no data entry for the protocol\n"
                    + protocol[0] + " " + protocol[1] + " " + protocol[2]
                    + "\nin the warehouse.\n"
                    + fileName + " file is not created." );
         return;
      }

      try {
         pw = new PrintWriter (
                   new BufferedWriter (
                       new FileWriter ( path + "dataset/" + fileName ) ) );
         //writing header
         pw.println( fileHeader );

         reader = new BufferedReader(
                    new FileReader( path + "output/LogFactTbl" ) );
         str = reader.readLine();
         str = reader.readLine();
         // while loop to compare and add into data set
         while( str != null && index < total ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               tID = tok.nextToken();
            }
            // time of the current entry
            d = new MyDate( tID );

            if( tok.hasMoreTokens() ) {
               String temp = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               prot = tok.nextToken();
               protID = Integer.parseInt( prot );
            }
            // compare
            if( protID == protocolID
                   && ( d.compareTo( d1 ) >= 0 ) && ( d.compareTo( d2 ) <= 0 ) ) {
               // adding to the data set
               pw.println( str );
               index++;
               // to create visit sequence
               MyRecord mr = new MyRecord( str );
               al.add( ( Object ) mr );
            }
            if( index >= 1 && index % 100 == 0 ) {
               System.out.println( "Till now size of the Data set: " + index + " ..." );
            }
            str = reader.readLine();
         }
         reader.close();
         //summery string
         String f = "";
         f = "\n\n*****Protocol based Dataset from Text File Warehouse*****"
           + "\nProtocol: " + protocol[0] + " " + protocol[1] + " " + protocol[2]
           + "\nStart Time: " + startTime
           + "\nEnd Time: " + endTime
           + "\nRequested size of the Data set: " + total
           + "\nActual size of the Data set: " + index;
         //writing summery
         pw.println( f );
         pw.close();
         // for a seperate file showing sequence of web page visited
         DataSetSequence dss = new DataSetSequence( al, fileName, f );
         dss.buildWebpageVisitSeq();
         //for display
         out.setText( f );
      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method createPathBasedDS will find pathID for the path from PathDimTbl.
     * Then it find all the entries for the pathID
     * from the LogFactTbl and compare the time stamp of the entries with
     * the staring time and ending time of the required data set.
     * If time stamp of the entry is equal to or more recent than the staring time
     * and equal to or earlier than the ending time, the entry is included into the data set
     *
     * @param       path                     string for path, file name, and
     *                                       file type
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException              for sql exception
     */

   public void createPathBasedDS( String[] fp ) {
      int index = 0;         // current number of entry in the data set
      int pathID = -1;       // pathID for the path, file name, and file type
      int paID = -1;         // pathID in int for the current entry
      String pa = "";        // pathID in String for the current entry
      String str = "";       // string to be added into data set
      String tID = "";       // timeID of the current entry at warehouse
      String hold = "";      // temporary storage
      MyDate d = null;              // current time
      MyDate d1 = null;             // start time
      MyDate d2 = null;             // end time
      ArrayList al;                 // to create visit sequence
      PrintWriter pw = null;
      BufferedReader reader = null;

      d1 = new MyDate( startTime );
      d2 = new MyDate( endTime );
      al = new ArrayList();

      // method call to find pathID of the path, file name, and file type
      pathID = getPathID( fp );
      // if there is no record of the fp, return with message
      if( pathID == -1 ) {
         out.setText( "\n\nThere is no data entry for the path\n"
                    + fp[0] + "/" + fp[1] + "." + fp[2]
                    + "\nin the warehouse.\n"
                    + fileName + " file is not created." );
         return;
      }

      try {
         pw = new PrintWriter (
                   new BufferedWriter (
                       new FileWriter ( path + "dataset/" + fileName ) ) );
         //writing header
         pw.println( fileHeader );

         reader = new BufferedReader(
                    new FileReader( path + "output/LogFactTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to compare and add into data set
         while( str != null && index < total ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               tID = tok.nextToken();
            }
            // time of the current entry
            d = new MyDate( tID );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               pa = tok.nextToken();
               paID = Integer.parseInt( pa );
            }
            // compare
            if( paID == pathID
                   && ( d.compareTo( d1 ) >= 0 ) && ( d.compareTo( d2 ) <= 0 ) ) {
               // adding to the data set
               pw.println( str );
               index++;
               // to create visit sequence
               MyRecord mr = new MyRecord( str );
               al.add( ( Object ) mr );
            }
            if( index >= 1 && index % 100 == 0 ) {
               System.out.println( "Till now size of the Data set: " + index + " ..." );
            }
            str = reader.readLine();
         }
         reader.close();
         //summery string
         String f = "";
         f = "\n\n*****Path based Dataset from Text File Warehouse*****"
           + "\nPath: " + fp[0] + "/" + fp[1] + "." + fp[2]
           + "\nStart Time: " + startTime
           + "\nEnd Time: " + endTime
           + "\nRequested size of the Data set: " + total
           + "\nActual size of the Data set: " + index;
         //writing summery
         pw.println( f );
         pw.close();
         // for a seperate file showing sequence of web page visited
         DataSetSequence dss = new DataSetSequence( al, fileName, f );
         dss.buildWebpageVisitSeq();
         //for display
         out.setText( f );
      } catch( FileNotFoundException fnf ) {
         System.out.println("DataSetControl: Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method getUserID will find userID for the ip address from UserDimTbl.
     *
     * @param       ipAddrs                  string for ip address
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException              for sql exception
     *
     * @return                               userID as int
     */

   public int getUserID( String ipAddrs ) {
      int userID = -1;              // userID of ipAddrs

      int uID = -1;                 // userID of the current entry
      String ip = "";               // ip address of the current entry
      String host = "";             // host name of the current entry
      String user = "";             // user of the current entry

      String str = "";              // current entry
      String hold = "";             // temporary storage
      BufferedReader reader = null; // reader

      try {
         reader = new BufferedReader(
                    new FileReader( path + "output/UserDimTbl" ) );
         str = reader.readLine();
         str = reader.readLine();
         // while loop to find a match
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               uID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               ip = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               host = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               user = tok.nextToken();
            }
            // compare
            if( ip != null && ip.equals( ipAddrs ) ) {
               userID = uID;
               break;
            }
            str = reader.readLine();
         }
         reader.close();
      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return userID;
   }

   /**
     * Method getProtocolID will find protocolID for the protocol from ProtocolDimTbl.
     *
     * @param       protocol                 array of string for protocol, method, and status
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException              for sql exception
     *
     * @return                               protocolID as int
     */

   public int getProtocolID( String[] protocol ) {
     int protocolID = -1;           // protocolID for protocol

     int protID = -1;               // protocolID for current entry
     String p = "";                 // http protocol of current entry
     String m = "";                 // method of current entry
     String s = "";                 // status of current entry

     String str = "";               // current entry
     String hold = "";              // temporary storage
     BufferedReader reader = null;  // reader

     try {
        reader = new BufferedReader(
                          new FileReader( path + "output/ProtocolDimTbl" ) );
        str = reader.readLine();
        str = reader.readLine();
        // while loop to find a match
        while( str != null ) {
           StringTokenizer tok = new StringTokenizer( str, " \t" );

           if( tok.hasMoreTokens() ) {
              hold = tok.nextToken();
              protID = Integer.parseInt( hold );
           }
           if( tok.hasMoreTokens() ) {
              p = tok.nextToken();
           }
           if( tok.hasMoreTokens() ) {
              m = tok.nextToken();
           }
           if( tok.hasMoreTokens() ) {
              s = tok.nextToken();
           }
           // compare
           if( p != null && p.equals( protocol[0] )
                  && m != null && m.equals( protocol[1] )
                      && s != null && s.equals( protocol[2] ) ) {
              protocolID = protID;
              break;
           }
           str = reader.readLine();
        }
        reader.close();
     } catch( FileNotFoundException fnf ) {
        System.out.println("Cannot locate input file! " + fnf.getMessage() );
     } catch( IOException ioe ){
        System.out.println( ioe.getMessage() );
     }
     return protocolID;
   }

   /**
     * Method getPathID will find pathID for the path, file name, and filetype
     * stored in arg from PathDimTbl.
     *
     * @param       arg                      array of string for path, file name, and file type
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException              for sql exception
     */

   public int getPathID( String[] arg ) {
      int pathID = -1;              // pathID of st

      int paID = -1;                // pathID for current entry
      String p = "";                // path of current entry
      String f = "";                // file name of current entry
      String t = "";                // file type of current entry

      String str = "";              // current entry
      String hold = "";             // temporary storage
      BufferedReader reader = null; // reader

      try {
         reader = new BufferedReader(
                    new FileReader( path + "output/PathDimTbl" ) );

         str = reader.readLine();
         str = reader.readLine();
         // while loop to find match
         while( str != null ) {
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               paID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               p = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               f = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               t = tok.nextToken();
            }

            // compare
            if( p != null && p.equals( arg[0] )
                 && f != null && f.equals( arg[1] )
                      && t != null && t.equals( arg[2] ) ) {
               pathID = paID;
               break;
            }
            str = reader.readLine();
         }
         reader.close();
      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return pathID;
   }
} // DataSetControl
