/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import java.io.*;
import javax.swing.*;

/**
  * This class is also a top level program which will coordinate to
  * create data set in text file from database warehouse, or text file warehouse
  *
* @author   MOHAMMED LIAKAT ALI   Student
*                                 School of Computer Science
*                                 University of Windsor
*
* @supervisor Dr. CHRISTIE EZEIFE Associate Professor
*                                 School of Computer Science
*                                 University of Windsor
*/

public class DataSetGenerator {
   private int count;               // will idetify type of the data set
   private JTextArea out;           // for display in the top most GUI

   private String fileName;         // output data set file name
   private int len;                 // size of the data set
   private String startTime;        // start time for the data set
   private String endTime;          // end time for the data set

   private String user;             // for data set of the user
   private String[] protocol;       // for data set of the protocol
   private String[] path;           // for data set of the path

   /**
     * The constructor will initiate variables
     */

   public DataSetGenerator( String fName, int l, int c, JTextArea o ) {
      count = c;
      out = o;

      fileName = fName;
      len = l;

      // the singleton object which will store start time, end time, user, protocol, and path
      Selector selector = Selector.Factory();
      // getting start time
      startTime = selector.getStartTime();
      // getting end time
      endTime = selector.getEndTime();

      // getting user
      user = selector.getUser();
      // getting protocol
      protocol = selector.getProtocol();
      // getting file path
      path = selector.getPath();
   }

   /**
     * Method operation will cordinate  to create data set in text file
     * from database warehouse, or text file warehouse based on the value of count
     */

   public void operation() {

      // if-else selection to call appropriate method
      if( count == 1 ) {
        timeBasedDSFromDB();
      } else if( count == 2 ){
        userBasedDSFromDB();
      } else if( count == 3 ) {
        protocolBasedDSFromDB();
      } else if( count == 4 ){
        pathBasedDSFromDB();
      } else if( count == 11 ) {
        timeBasedDSFromTF();
      } else if( count == 12 ){
        userBasedDSFromTF();
      } else if( count == 13 ) {
        protocolBasedDSFromTF();
      } else if( count == 14 ){
        pathBasedDSFromTF();
      }
   }

   /**
     * Method timeBasedDSFromDB will create an object of DataSetModel class
     * and call createTimeBasedDS method
     * to create data set in text file from database warehouse
     */

   public void timeBasedDSFromDB() {
      System.out.println( "\nStart of time based data set from database" );
      DataSetModel ds = new DataSetModel( fileName, startTime, endTime, len, out );
      ds.createTimeBasedDS();
      ds.closeConnection();
      System.out.println( "End of time based data set from database\n" );
   }

   /**
     * Method userBasedDSFromDB will create an object of DataSetModel class
     * and call createUserBasedDS method
     * to create data set in text file from database warehouse
     */

   public void userBasedDSFromDB() {
      System.out.println( "\nStart of user based data set from database" );
      DataSetModel ds = new DataSetModel( fileName, startTime, endTime, len, out );
      ds.createUserBasedDS( user );
      ds.closeConnection();
      System.out.println( "End of user based data set from database\n" );
   }

   /**
     * Method protocolBasedDSFromDB will create an object of DataSetModel class
     * and call createProtocolBasedDS method
     * to create data set in text file from database warehouse
     */

   public void protocolBasedDSFromDB() {
      System.out.println( "\nStart of protocol based data set from database" );
      DataSetModel ds = new DataSetModel( fileName, startTime, endTime, len, out );
      ds.createProtocolBasedDS( protocol );
      ds.closeConnection();
      System.out.println( "End of protocol based data set from database\n" );
   }

   /**
     * Method pathBasedDSFromDB will create an object of DataSetModel class
     * and call createPathBasedDS method
     * to create data set in text file from database warehouse
     */

   public void pathBasedDSFromDB() {
      System.out.println( "\nStart of path based data set from database" );
      DataSetModel ds = new DataSetModel( fileName, startTime, endTime, len, out );
      ds.createPathBasedDS( path );
      ds.closeConnection();
      System.out.println( "End of path based data set from database\n" );
   }

   /**
     * Method timeBasedDSFromTF will create an object of DataSetControl class
     * and call createTimeBasedDS method
     * to create data set in text file from text files warehouse
     */

   public void timeBasedDSFromTF() {
      System.out.println( "\nStart of time based data set from text file" );
      DataSetControl ds = new DataSetControl( fileName, startTime, endTime, len, out );
      ds.createTimeBasedDS();
      System.out.println( "End of time based data set from text file\n" );
   }

   /**
     * Method userBasedDSFromTF will create an object of DataSetControl class
     * and call createUserBasedDS method
     * to create data set in text file from text files warehouse
     */

   public void userBasedDSFromTF() {
      System.out.println( "\nStart of user based data set from text file" );
      DataSetControl ds = new DataSetControl( fileName, startTime, endTime, len, out );
      ds.createUserBasedDS( user );
      System.out.println( "End of user based data set from text file\n" );
   }

   /**
     * Method protocolBasedDSFromTF will create an object of DataSetControl class
     * and call createProtocolBasedDS method
     * to create data set in text file from text files warehouse
     */

   public void protocolBasedDSFromTF() {
      System.out.println( "\nStart of protocol based data set from text file" );
      DataSetControl ds = new DataSetControl( fileName, startTime, endTime, len, out );
      ds.createProtocolBasedDS( protocol );
      System.out.println( "End of protocol based data set from text file\n" );
   }

   /**
     * Method pathBasedDSFromTF will create an object of DataSetControl class
     * and call createPathBasedDS method
     * to create data set in text file from text files warehouse
     */

   public void pathBasedDSFromTF() {
      System.out.println( "\nStart of path based data set from text file" );
      DataSetControl ds = new DataSetControl( fileName, startTime, endTime, len, out );
      ds.createPathBasedDS( path );
      System.out.println( "End of path based data set from text file\n" );
   }
} // DataSetGenerator
