/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import javax.swing.*;
import java.io.*;
import java.util.*;
import java.sql.*;

/**
  * This program will implement all related activities for creating
  * data set in text file from the database warehouse.
  * It will create the following data sets:
  *    1) time based data set
  *    2) user based data set
  *    3) protocol based data set
  *    4) path based data set
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class DataSetModel {
   private JTextArea out;          // for display
   private Connection con = null;  // represent connection with the database

   private int total;              // data set size
   private String fileName;        // data set file name
   private String fileHeader;      // store header string for the data sets

   private String startTime;          // data set start time
   private String endTime;            // data set end time

   private String path;               // file path

   /**
    * The constructor will establish connection with the database
     * specified in the configaration file 'configFile.INI'
     * and initialize variables
     *
     * @exception   ClassNotFoundException         if class specified by driver cannot be located
     *
     * @exception   SQLException                   for sql exception
     */

   public DataSetModel( String fName, String st, String et, int t, JTextArea o ) {
      out = o;

      total = t;
      fileName = fName;

      startTime = st;
      endTime = et;
      // the singleton object which will store file path path, driver, and connection names
      Config config = Config.Factory();
      //getting file path name
      path = config.getPath();
      //getting driver name
      String driver = config.getDriver();
      //getting connection name
      String connection = config.getConnection();

      // the singleton object which will store  database user id and password
      DBLogin login = DBLogin.Factory();
      // getting database user id
      String userID = login.getUserID();
      // getting database password
      String password = login.getPassword();

      // creating header of the dataset
      String s = "";
      s += "timeID ";
      while( s.length() < 30 ) {
         s += " ";
      }
      s += "userID ";
      while( s.length() < 40 ) {
         s += " ";
      }
      s += "protocolID ";
      while( s.length() < 55 ) {
         s += " ";
      }
      s += "pathID ";
      while( s.length() < 65 ) {
         s += " ";
      }
      s += "sessionID ";
      while( s.length() < 80 ) {
         s += " ";
      }
      s += "nbyte";
      // getting file header
      fileHeader = s;

      try {
         // getting reference
         Class.forName ( driver );
         // getting connection with the database
         con = DriverManager.getConnection( connection, userID, password );
      } catch ( ClassNotFoundException cnf ) {
         cnf.printStackTrace();
      } catch ( SQLException sqle ) {
         sqle.printStackTrace();
      }
   }

   /**
    * Method closeConnection will disconnect the program from the database
     * specified in the configaration file 'configFile.INI'
     *
     * @exception   SQLException                   for sql exception
     */

   public void closeConnection() {
      try {
         con.close();
      } catch (SQLException sqle ) {
         sqle.printStackTrace();
      }
   }

   /**
     * Method createTimeBasedDS will check time stamp of all the enrtries of the LogFactTbl
     * and compare with the staring time and ending time of the required data set.
     * If time stamp of the entry is equal to or more recent than the staring time
     * and equal to or earlier than the ending time, the entry is included into the data set
     *
     * @exception   SQLException             for sql exception
     *
     * @exception   FileNotFoundException    if output file can not be located
     *
     * @exception   IOException              for io exception
     */

   public void createTimeBasedDS() {
      int index = 0;                // current number of entry in the data set
      String str = "";              // string to be added into data set
      String tID = "";              // timeID of the current entry at warehouse

      MyDate d = null;              // current time
      MyDate d1 = null;             // start time
      MyDate d2 = null;             // end time
      ArrayList al;                 // to create visit sequence
      PrintWriter pw = null;

      Statement stmt = null;
      ResultSet result = null;
      String qry = "";

      d1 = new MyDate( startTime );
      d2 = new MyDate( endTime );
      al = new ArrayList();

      try {
         pw = new PrintWriter (
                   new BufferedWriter (
                       new FileWriter ( path + "dataset/" + fileName ) ) );
         // header of the data set
         pw.println( fileHeader  );

         stmt = con.createStatement();
         qry = "SELECT * from LogFactTbl";
         result = stmt.executeQuery( qry );
         // while loop to compare and add into data set
         while ( result != null && result.next() && index < total ) {
            tID = result.getString ( 1 );
            // time of the current entry
            d = new MyDate( tID );
            // compare
            if( ( d.compareTo( d1 ) >= 0 ) && ( d.compareTo( d2 ) <= 0 ) ) {
               //creating a record to be added to the data set
               str = tID + " ";
               while( str.length() < 30 ) {
                  str += " ";
               }
               str += result.getInt( 2 ) + " ";
               while( str.length() < 40 ) {
                  str += " ";
               }
               str += result.getInt( 3 ) + " ";
               while( str.length() < 55 ) {
                  str += " ";
               }
               str += result.getInt( 4 ) + " ";
               while( str.length() < 65 ) {
                  str += " ";
               }
               str += result.getInt( 5 ) + " ";
               while( str.length() < 80 ) {
                  str += " ";
               }
               str += result.getInt( 6 );
               // adding to the data set
               pw.println( str );
               index++;
               // to create visit sequence
               MyRecord mr = new MyRecord( str );
               al.add( ( Object ) mr );
            }
            if( index >= 1 && index % 100 == 0 ) {
               System.out.println( "Till now size of the Data set: " + index + " ..." );
            }
         }
         result.close();
         stmt.close();
         //summery string
         String temp = "";
         temp = "\n\n*****Time based Dataset from Database Warehouse*****"
              + "\nStart Time: " + startTime
              + "\nEnd Time: " + endTime
              + "\nRequested size of the Data set: " + total
              + "\nActual size of the Data set: " + index;
         //writing summery
         pw.println( temp );
         pw.close();
         // for a seperate file showing sequence of web page visited
         DataSetSequence dss = new DataSetSequence( al, fileName, temp );
         dss.buildWebpageVisitSeq();
         //for display
         out.setText( temp );
      } catch( SQLException sqle ) {
          sqle.printStackTrace();
      } catch( FileNotFoundException fnf ) {
         System.out.println( "Cannot locate output file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method createUserBasedDS will find userID for the user from UserDimTbl.
     * Then it will find all the entries for the userID from the LogFactTbl and
     * compare the time stamp of the entries with the staring time and ending time of the required data set.
     * If time stamp of the entry is equal to or more recent than the staring time
     * and equal to or earlier than the ending time, the entry is included into the data set
     *
     * @param       user                     ip address of the user
     *
     * @exception   SQLException             for sql exception
     *
     * @exception   FileNotFoundException    if file cannot be located
     *
     * @exception   IOException              for io exception
     */

   public void createUserBasedDS( String user ) {
      int index = 0;               // current number of entry in the data set
      int userID = -1;             // userID for the user
      String str = "";             // string to be added into data set
      String tID = "";             // timeID of the current entry at warehouse

      MyDate d = null;             // current time
      MyDate d1 = null;            // start time
      MyDate d2 = null;            // end time
      ArrayList al;                // to create visit sequence
      PrintWriter pw = null;

      Statement stmt = null;
      ResultSet result = null;
      String qry = "";

      d1 = new MyDate( startTime );
      d2 = new MyDate( endTime );
      al = new ArrayList();

      // method call to get userID for the user
      userID = getUserID( user );
      // if there is no record of the user, return with message
      if( userID == -1 ) {
         out.setText( "\n\nThere is no data entry for the user:\n" + user
                    + "\nin the warehouse.\n"
                    + fileName + " file is not created." );
         return;
      }

      try {
         pw = new PrintWriter (
                   new BufferedWriter (
                        new FileWriter ( path + "dataset/" + fileName ) ) );
         // header of the data set
         pw.println( fileHeader  );

         stmt = con.createStatement();
         qry = "SELECT * from LogFactTbl WHERE userID = " + userID;
         result = stmt.executeQuery( qry );
         // while loop to compare and add into data set
         while ( result != null && result.next() && index < total ) {
            tID = result.getString ( 1 );
            // time of the current entry
            d = new MyDate( tID );
            // compare
            if( ( d.compareTo( d1 ) >= 0 ) && ( d.compareTo( d2 ) <= 0 ) ) {
               //creating a record to be added to the data set
               str = tID + " ";
               while( str.length() < 30 ) {
                  str += " ";
               }
               str += result.getInt( 2 ) + " ";
               while( str.length() < 40 ) {
                  str += " ";
               }
               str += result.getInt( 3 ) + " ";
               while( str.length() < 55 ) {
                  str += " ";
               }
               str += result.getInt( 4 ) + " ";
               while( str.length() < 65 ) {
                  str += " ";
               }
               str += result.getInt( 5 ) + " ";
               while( str.length() < 80 ) {
                  str += " ";
               }
               str += result.getInt( 6 );
               // adding to the data set
               pw.println( str );
               index++;
               // to create visit sequence
               MyRecord mr = new MyRecord( str );
               al.add( ( Object ) mr );
            }
            if( index >= 1 && index % 100 == 0 ) {
               System.out.println( "Till now size of the Data set: " + index + " ..." );
            }
         }
         result.close();
         stmt.close();
         //summery string
         String temp = "";
         temp = "\n\n*****User based Dataset from Database Warehouse*****"
              + "\nUser: " + user
              + "\nStart Time: " + startTime
              + "\nEnd Time: " + endTime
              + "\nRequested size of the Data set: " + total
              + "\nActual size of the Data set: " + index;
         //writing summery
         pw.println( temp );
         pw.close();
         // for a seperate file showing sequence of web page visited
         DataSetSequence dss = new DataSetSequence( al, fileName, temp );
         dss.buildWebpageVisitSeq();
         //for display
         out.setText( temp );
      } catch (SQLException sqle ) {
          sqle.printStackTrace();
      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate output file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method createProtocolBasedDS will find protocolID for
     * the protocol, method, and status from ProtocolDimTbl.
     * Then it will find all the entries for the protocolID
     * from the LogFactTbl and compare the time stamp of the entries with
     * the staring time and ending time of the required data set.
     * If time stamp of the entry is equal to or more recent than the staring time
     * and equal to or earlier than the ending time, the entry is included into the data set
     *
     * @param       protocol                 string for protocol, method, and
     *                                       status
     *
     * @exception   SQLException             for sql exception
     *
     * @exception   FileNotFoundException    for sql exception
     *
     * @exception   IOException              for sql exception
     */


   public void createProtocolBasedDS( String[] protocol ) {
      int index = 0;         // current number of entry in the data set
      int protocolID = -1;   // protocolID for the protocol, method, and status
      String str = "";       // string to be added into data set
      String tID = "";       // timeID of the current entry at warehouse

      MyDate d = null;              // current time
      MyDate d1 = null;             // start time
      MyDate d2 = null;             // end time
      ArrayList al;                 // to create visit sequence
      PrintWriter pw = null;

      Statement stmt = null;
      ResultSet result = null;
      String qry = "";

      d1 = new MyDate( startTime );
      d2 = new MyDate( endTime );
      al = new ArrayList();

      // method call to find protocolID of the protocol, method, and status
      protocolID = getProtocolID( protocol );
      // if there is no record of the protocol, return with message
      if( protocolID == -1 ) {
         out.setText( "\n\nThere is no data entry for the protocol\n"
                    + protocol[0] + " " + protocol[1] + " " + protocol[2]
                    + "\nin the warehouse.\n"
                    + fileName + " file is not created." );
         return;
      }

      try {
         pw = new PrintWriter (
                   new BufferedWriter (
                       new FileWriter ( path + "dataset/" + fileName ) ) );
         // header of the data set
         pw.println( fileHeader  );

         stmt = con.createStatement();
         qry = "SELECT * from LogFactTbl WHERE protocolID = " + protocolID;
         result = stmt.executeQuery( qry );
         // while loop to compare and add into data set
         while ( result != null && result.next() && index < total ) {
            tID = result.getString ( 1 );
            // time of the current entry
            d = new MyDate( tID );
            // compare
            if( ( d.compareTo( d1 ) >= 0 ) && ( d.compareTo( d2 ) <= 0 ) ) {
               //creating a record to be added to the dat set
               str = tID + " ";
               while( str.length() < 30 ) {
                  str += " ";
               }
               str += result.getInt( 2 ) + " ";
               while( str.length() < 40 ) {
                  str += " ";
               }
               str += result.getInt( 3 ) + " ";
               while( str.length() < 55 ) {
                  str += " ";
               }
               str += result.getInt( 4 ) + " ";
               while( str.length() < 65 ) {
                  str += " ";
               }
               str += result.getInt( 5 ) + " ";
               while( str.length() < 80 ) {
                  str += " ";
               }
               str += result.getInt( 6 );
               // adding to the data set
               pw.println( str );
               index++;
               // to create visit sequence
               MyRecord mr = new MyRecord( str );
               al.add( ( Object ) mr );
            }
            if( index >= 1 && index % 100 == 0 ) {
               System.out.println( "Till now size of the Data set: " + index + " ..." );
            }
         }
         result.close();
         stmt.close();
         //summery string
         String temp = "";
         temp = "\n\n*****Protocol based Dataset from Database Warehouse*****"
              + "\nProtocol: " + protocol[0] + " " + protocol[1] + " " + protocol[2]
              + "\nStart Time: " + startTime
              + "\nEnd Time: " + endTime
              + "\nRequested size of the Data set: " + total
              + "\nActual size of the Data set: " + index;
         //writing summery
         pw.println( temp );
         pw.close();
         // for a seperate file showing sequence of web page visited
         DataSetSequence dss = new DataSetSequence( al, fileName, temp );
         dss.buildWebpageVisitSeq();
         //for display
         out.setText( temp );
      } catch (SQLException sqle ) {
          sqle.printStackTrace();
      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method createPathBasedDS will find pathID for
     * the path, file name, and filetype from PathDimTbl.
     * Then it find all the entries for the pathID
     * from the LogFactTbl and compare the time stamp of the entries with
     * the staring timeand ending time of the required data set.
     * If time stamp of the entry is equal to or more recent than the staring time
     * and equal to or earlier than the ending time, the entry is included into the data set
     *
     * @param       fp                      array of string for path, file name, and file type
     *
     * @exception   SQLException             for sql exception
     *
     * @exception   FileNotFoundException    if file can not be located
     *
     * @exception   IOException              for io exception
     */

   public void createPathBasedDS( String[] fp ) {
      int index = 0;         // current number of entry in the data set
      int pathID = -1;       // pathID for the path, file name, and file type
      String str = "";       // string to be added into data set
      String tID = "";       // timeID of the current entry at warehouse

      MyDate d = null;              // current time
      MyDate d1 = null;             // start time
      MyDate d2 = null;             // end time
      ArrayList al;                 // to create visit sequence
      PrintWriter pw = null;

      Statement stmt = null;
      ResultSet result = null;
      String qry = "";

      d1 = new MyDate( startTime );
      d2 = new MyDate( endTime );
      al = new ArrayList();

      // method call to find pathID of the path, file name, and file type
      pathID = getPathID( fp );
      // if there is no record of the fp, return with message
      if( pathID == -1 ) {
         out.setText( "\n\nThere is no data entry for the path:\n"
                    + fp[0] + "/" + fp[1] + "." + fp[2]
                    + "\nin the warehouse.\n"
                    + fileName + " file is not created." );
         return;
      }

      try {
         pw = new PrintWriter (
                   new BufferedWriter (
                       new FileWriter ( path + "dataset/" + fileName ) ) );
         // header of the data set
         pw.println( fileHeader  );

         stmt = con.createStatement();
         qry = "SELECT * from LogFactTbl WHERE pathID = " + pathID;
         result = stmt.executeQuery( qry );
         // while loop to compare and add into data set
         while ( result != null && result.next() && index < total ) {
            tID = result.getString ( 1 );
            // time of the current entry
            d = new MyDate( tID );
            // compare
            if( ( d.compareTo( d1 ) >= 0 ) && ( d.compareTo( d2 ) <= 0 ) ) {
               //creating a record to be added to tha data set
               str = tID + " ";
               while( str.length() < 30 ) {
                  str += " ";
               }
               str += result.getInt( 2 ) + " ";
               while( str.length() < 40 ) {
                  str += " ";
               }
               str += result.getInt( 3 ) + " ";
               while( str.length() < 55 ) {
                  str += " ";
               }
               str += result.getInt( 4 ) + " ";
               while( str.length() < 65 ) {
                  str += " ";
               }
               str += result.getInt( 5 ) + " ";
               while( str.length() < 80 ) {
                  str += " ";
               }
               str += result.getInt( 6 );
               // adding to the data set
               pw.println( str );
               index++;
               // to create visit sequence
               MyRecord mr = new MyRecord( str );
               al.add( ( Object ) mr );
            }
            if( index >= 1 && index % 100 == 0 ) {
               System.out.println( "Till now size of the Data set: " + index + " ..." );
            }
         }
         result.close();
         stmt.close();
         //summery string
         String temp = "";
         temp = "\n\n*****Path based Dataset from Database Warehouse*****"
              + "\nPath: " + fp[0] + "/" + fp[1] + "." + fp[2]
              + "\nStart Time: " + startTime
              + "\nEnd Time: " + endTime
              + "\nRequested size of the Data set: " + total
              + "\nActual size of the Data set: " + index;
         //writingsummery
         pw.println( temp );
         pw.close();
         // for a seperate file showing sequence of web page visited
         DataSetSequence dss = new DataSetSequence( al, fileName, temp );
         dss.buildWebpageVisitSeq();
         //for display
         out.setText( temp );
      } catch (SQLException sqle ) {
          sqle.printStackTrace();
      } catch( FileNotFoundException fnf ) {
         System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method getUserID will find userID for the ip address from UserDimTbl.
     *
     * @param       ipAddrs                  string for IP address
     *
     * @exception   SQLException             for sql exception
     *
     * @return                               userID as int
     */

   public int getUserID( String ipAddrs ) {
      int userID = -1;              // userID of ipAddrs
      String ip = "";

      Statement stmt = null;
      ResultSet result = null;
      String qry = "";

      try {
         stmt = con.createStatement();
         qry = "SELECT * FROM UserDimTbl";
         result = stmt.executeQuery( qry );
         // finding userID of the user
         while( result.next() ) {
            ip = result.getString( 2 );
            // compare
            if( ip.equals( ipAddrs ) ) {
               userID = result.getInt ( 1 );
               break;
            }
         }
         result.close();
         stmt.close();
     } catch( SQLException sqle ) {
        sqle.printStackTrace();
     }
     return userID;
   }

   /**
     * Method getProtocolID will find protocolID for the protocol from ProtocolDimTbl.
     *
     * @param       protocol                 array of string for protocol, method, and status
     *
     * @exception   SQLException             for sql exception
     *
     * @exception   FileNotFoundException    if file can not be located
     *
     * @exception   IOException              for io exception
     *
     * @return                               protocolID as int
     */

   public int getProtocolID( String[] protocol ) {
     int protID = -1;             // protocolID for protocol

     String p = "";              // http protocol of current entry
     String m = "";              // method of current entry
     int s = -1;                  // status of current entry

     Statement stmt = null;
     ResultSet result = null;
     String qry = "";

     int s1 = Integer.parseInt( protocol[2] );

     try {
        stmt = con.createStatement();
        qry = "SELECT * FROM ProtocolDimTbl";
        result = stmt.executeQuery( qry );
        // while loop to find a match
        while ( result.next() ) {
           p = result.getString( 2 );
           m = result.getString( 3 );
           s = result.getInt( 4 );
           // compare
           if( p != null && p.equals( protocol[0] )
                  && m != null && m.equals( protocol[1] )
                      && s != -1 && s == s1 ) {
              protID = result.getInt ( 1 );
              break;
           }

        }
        result.close();
        stmt.close();
     } catch( SQLException sqle ) {
        sqle.printStackTrace();
     }
     return protID;
   }

   /**
     * Method getPathID will find pathID for the path, file name, and filetype
     * stored in arg from PathDimTbl.
     *
     * @param       arg                      string array for path, file name, and file type
     *
     * @exception   SQLException             for sql exception
     *
     * @exception   FileNotFoundException    if file can not be located
     *
     * @exception   IOException              for io exception
     */

   public int getPathID( String[] arg ) {
      int paID = -1;            // pathID for st

      boolean flag = false;

      String p = "";            // path of current entry
      String f = "";            // file name of current entry
      String t = "";            // file type of current entry

      Statement stmt = null;
      ResultSet result = null;
      String qry = "";

      try {
         stmt = con.createStatement();
         qry = "SELECT * FROM PathDimTbl";
         result = stmt.executeQuery( qry );
         // while loop to find match
         while ( result.next() ) {
            p = result.getString( 2 );
            f = result.getString( 3 );
            t = result.getString( 4 );

            // compare
            if( p != null && p.equals( arg[0] )
                  && f != null && f.equals( arg[1] )
                     && t != null && t.equals( arg[2] ) ) {

               paID = result.getInt ( 1 );
               break;
            }
         }
         result.close();
         stmt.close();
      } catch( SQLException sqle ) {
         sqle.printStackTrace();
      }
      return paID;
   }
} // DataSetModel
