/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.sql.*;

/**
  * This program will:
  *    1) get host name corresponding to ip address from DNS
  *    2) update the UserDimTbl with the host name
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class HostModel {
   private Connection con = null;    // represent connection with the database

   /**
     * The constructor will establish connection with the database
     * specified in the configaration file 'configFile.INI'
     *
     * @exception   ClassNotFoundException         if class specified by driver cannot be located
     *
     * @exception   SQLException                   for sql exception
     */

   public HostModel () {
      // the singleton object which will store driver and connection names for database access
      Config config = Config.Factory();
      // getting driver
      String driver = config.getDriver();
      // getting connection
      String connection = config.getConnection();

      // the singleton object which will store user id and password for database access
      DBLogin login = DBLogin.Factory();
      // getting user id
      String userID = login.getUserID();
      // getting password
      String password = login.getPassword();

      try {
         // getting reference
         Class.forName ( driver );
         // getting connection
         con = DriverManager.getConnection( connection, userID, password );
      } catch ( ClassNotFoundException cnf ) {
         cnf.printStackTrace();
      } catch ( SQLException sqle ) {
         sqle.printStackTrace();
      }
   }

   /**
     * Method closeConnection will disconnect the program from the database
     * specified in the configaration file 'configFile.INI'
     *
     * @exception   SQLException                   for sql exception
     */

   public void closeConnection() {
      try {
         con.close();
      } catch ( SQLException sqle ) {
         sqle.printStackTrace();
      }
   }

   /**
     * Method updateDBUserDimTbl will check host name of UserDimTble.
     * If host name is not found, it will get host name corresponding ip address from DNS
     * and update the table of the warehouse in the database
     *
     * @exception   SQLException            for sql exception
     *
     * @return                              number of record updated
     */

   public int updateDBUserDimTbl() {
     int count = 0;                  // number of host name updates
     int uID = -1;                   // user id of the query
     String ip = "";                 // IP address of the query
     String host = "";               // host name of the query
     String qry = "";                // query string

     Statement stmtU = null;         // upate Statement
     Statement stmtQ = null;         // query Statement
     ResultSet result = null;        // query ResultSet

     try {
        stmtU = con.createStatement();                 // getting update Statement reference
        stmtQ = con.createStatement();                 // getting query Statement reference

        qry = "SELECT * From UserDimTbl";
        result = stmtQ.executeQuery( qry );            // getting query ResultSet reference
        // while loop to check all the entries of the UserDimTble
        // if they have host names corresponding to IP address
        while( result != null && result.next() ) {
           uID = result.getInt( 1 );
           ip = result.getString ( 2 );
           host = result.getString ( 3 );

           int index = -1;
           // if a entry has no host name
           if( host.equals( "-" ) ) {
              // getting host name corresponding to the IP address
              String hostName = getHostName( ip );
              String update = "UPDATE UserDimTbl "
                                + "SET hostName = \'" + hostName
                                + "\' WHERE userID = " + uID;
              // updating
              index = stmtU.executeUpdate( update );
              // to keep count
              if( index >= 1 ) {
                 count++;
              }
           }
           if( count >= 1 && count % 100 == 0 ) {
              System.out.println( "Till now " + count + " host names are updated..." );
           }
        }
        result.close();
        stmtQ.close();
        stmtU.close();
     } catch ( SQLException sqle ) {
        sqle.printStackTrace();
        count = -1;
     }
     return count;
   }

   /**
     * Method getHostName will access DNS for host name correseponding to the IPAddress
     *
     * @param       ipAddrs                   IP Address
     *
     * @exception   UnknownHostException      if host is not known
     *
     * @exception   IOException               for io exceptions
     *
     * @return                                Host name
     */

   public String getHostName( String ipAddrs ) {
      String host = "-";
      try {
         // getting reference to InetAddress object
         InetAddress ia = InetAddress.getByName( ipAddrs );
         // method call
         host = ia.getHostName();
      } catch( UnknownHostException uh ) {
         System.out.println("HostModel: Cannot locate host! " + uh.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return host;
   }
} // HostModel
