/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import javax.swing.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.regex.*;

/**
  * This program will:
  *    1) find last access time of the stored records in text files warehouse
  *    2) read the copy of the Web Server Access Log file from input directory line by line
  *    3) send the record to store in the warehouse,
  *       a)if date stamp of the access record is same as or more recent than the last access time
  *       b)the record is not in the warehouse, and
  *       c)the record contains valid data
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */


public class TFDataCollector {
   private JTextArea out;           // for display
   private MyDate lastAccessTime;   // latest record in the database warehouse before the present run
   private String path;             // file path
   private DataManager dm;          // for cleaning and identifying data from access record


   /**
     * The constructor will initiate out,lastAcceseeTime, path, and dm varibles
     */

   public TFDataCollector( JTextArea o ) {
      out = o;
      // getting a Control object to find last access time
      DataCollectorControl con = new DataCollectorControl();
      // lastAccessTime will get latest record stored in text file warehouse before the present run of the program
      lastAccessTime = con.getLastAccessTime();
      // getting a Config singleton object to find file path
      Config config = Config.Factory();
      path = config.getPath();
      // initializing the DataManeger object
      dm = null;
   }

   /**
     * Method processData will
     *    1) read all access records from the input file line by line (one record at a time).
     *    2) clean the data
     *    3) check if the record has valid data
     *
     * If a record has valid data, it will
     *    1) get cleaned and processed data
     *    2) initialized currentTime variable
     *    3) comapre lastaccessTime with currentTime.
     *
     * If currentTime is equals to or more recent than the lastaccessTime,
     * it will check if the record is in the warehouse. If the record is not found
     * in the warehouse, it will send the record to be stored in the data warehouse.
     *
     * @exception   FileNotFoundException   if input file cannot be located
     *
     * @exception   IOException             for io exceptions
     */

   public void processData() {
      int timeIndex = 0;           // will be less, equals, or more than zero
      int count = -1;              // will indicate if present record is stored or not
      int index = 0;               // will show how many records are stored
      int total = 0;               // will show how many records are accessed
      boolean flag = false;        // will indicate if record has valid data or not
      boolean inStoreFlag = false; // will indicate if the record is already in the warehouse or not
      String str = "";             // will store present record
      String[] arg;                // will store a record of processed and valid data

      BufferedReader reader;
      try {
         reader = new BufferedReader(
                    new FileReader( path + "input/accessLog" ) );

         str = reader.readLine();
         // while loop to read and process all the records of the input file
         while( str != null ) {
            // will retain the DataManeger object for data cleaning
            if( dm == null ) {
               dm = new DataManager( str );
            } else {
               dm.setRawData( str );
            }
            // checking validity of data
            flag = dm.isValidData();
            if( flag ) {
               // getting cleaned and valid data
               arg = dm.getProcessedData();
               // getting currentTime from the time stamp
               MyDate currentTime = new MyDate( arg[3] + "/" + arg[4] );
               // if-else selection to find timeIndex for the current record
               if( lastAccessTime != null ) {
                  timeIndex = currentTime.compareTo( lastAccessTime );
               }
               // if currentTime is equals to or more recent than the lastAccessTime
               if( timeIndex >= 0 ) {
                 // checking if the record is in the data warehouse
                  inStoreFlag = isInStore( arg );
                  if( !inStoreFlag ) {
                     // sending the record for storage
                     count = sendToWarehouse( arg );
                  }
               }
            }
            // update number of records stored
            if( count != -1 ) {
               index++;
               count = -1;
            }
            // update number of records accessed
            total++;
            if( total % 100 == 0 ) {
               System.out.println( "Till now " + total + " lines are accessed \nand "
                                   + index + " lines are stored..." );
            }
            str = reader.readLine();
         }
         reader.close();
      } catch( FileNotFoundException fnf ) {
         System.out.println("TFDataCollector: Cannot locate input file! "
                                                       + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      String s = "\n\n\nTotal " + total + " lines are accessed "
                 + "from Web Access log file.                \n"
                 + "Total " + index
                 + " new record(s) are stored to text files";
      System.out.println( s );
      // for display
      out.setText( s );
   }

   /**
     * Method isInStore will create an object of Control class
     * and use the object's isInStore method to find
     * if the record represented by the String array str is in the data warehouse.
     *
     * @param       str                     str is an array of String
     *
     * @return                              boolean
     */

   public boolean isInStore( String[] str ) {
      boolean flag = false;

      DataCollectorControl con = new DataCollectorControl();
      flag = con.isInStore( str );

      return flag;
   }

   /**
     * Method sendToWarehouse will
     *    1) send IPAddress, Host Name, and User to store in UserDimTbl
     *    2) create timeID
     *    3) send Protocol, Method, and Status to store in ProtocolDimTbl
     *    4) send Path, File, and FileType to store in PathDimTbl
     *    5) send timeID, userID, protocolID, pathID, and no. of Byte to store in LogFactTbl
     *
     *
     * @param       arg                     arg is an array of String
     *
     * @return                              int
     */

   public int sendToWarehouse ( String[] arg ) {
      String timeID = "";
      int protocolID = -1;
      int userID = -1;
      int pathID = -1;
      int count = -1;

      // method call to store data
      //IP address is stored in arg[0] );
      //Host Name is stored in arg[1] );
      //User is stored in arg[2] );
      userID = insertToUserDimTbl( arg[0], arg[1], arg[2] );

      // creating timeID
      //Timestamp is stored in  arg[3] );
      //offset is stored in arg[4] );
      timeID = arg[3] + "/" + arg[4];

      // method call to store data
      //Method is stored in arg[5] );
      //Protocol is stored in arg[9] );
      //Status is stored in arg[10] );
      protocolID = insertToProtocolDimTbl( arg[9], arg[5], arg[10] );

      // method call to store data
      //Path is stored in arg[6]
      //File is stored in arga[7]
      //File Type is stored in arg[8]
      pathID = insertToPathDimTbl( arg[6], arg[7], arg[8] );


      //number of Bytes are stored in arg[11] );
      if( timeID != "" && userID != -1 && protocolID != -1 && pathID != -1 ) {
        int nbyte;

        // if-else selection to find number of byte
        if( !arg[11].equals( "-" ) ) {
           nbyte = Integer.parseInt( arg[11] );
        } else {
           nbyte = 0;
        }
         // method call to store data
         count = insertToLogFactTbl( timeID, userID, protocolID, pathID, nbyte );
      }
      return count;
   }

   /**
     * Method insertToUserDimTbl will create an object of Control class
     * and use the object's insertToUserDimTbl method to store the arguments
     * to UserDimTbl.
     *
     * @param       ip                     ip is IPAddress
     *
     * @param       hostName               host name of the record
     *
     * @param       userName               user name of the record
     *
     * @return                             int
     */

   public int insertToUserDimTbl( String ip,
                                  String hostName, String userName ) {
      int count = -1;

      DataCollectorControl con = new DataCollectorControl();
      count = con.insertToUserDimTbl( ip, hostName, userName );

      return count;
   }

   /**
     * Method insertToProtocolDimTbl will create an object of Control class
     * and use the object's insertToProtocolDimTbl method to store
     * the arguments to ProtocolDimTbl.
     *
     * @param       prot                   protocol name
     *
     * @param       mt                     method name
     *
     * @param       stStr                  status
     *
     * @return                             int
     */

   public int insertToProtocolDimTbl( String prot, String mt, String stStr ) {
      int count = -1;

      DataCollectorControl con = new DataCollectorControl();
      count = con.insertToProtocolDimTbl( prot, mt, stStr );

      return count;
   }

   /**
     * Method insertToPathDimTbl will create an object of Control class
     * and use the object's insertToPathDimTbl method to store
     * the arguments to PathDimTbl.
     *
     * @param       p                    path
     *
     * @param       f                    file name
     *
     * @param       t                    file type
     *
     * @return                           int
     */

   public int insertToPathDimTbl( String p, String f, String t ) {
      int count = -1;

      DataCollectorControl con = new DataCollectorControl();
      count = con.insertToPathDimTbl( p, f, t );

      return count;
   }

   /**
     * Method insertToLogFactTbl will create an object of Control class
     * and use the object's insertToLogFactTbl method to store
     * the arguments to LogFactTbl.
     *
     * @param       timeID          logical key of time stamp of the record
     *
     * @param       userID          primary key of the entry in UserDimTbl
     *
     * @param       protocolID      primary key of the entry in ProtocolDimTbl
     *
     * @param       pathID          primary key of the entry in PathDimTbl
     *
     * @param       b               number of bytes accessed
     *
     * @return                      int
     */

   public int insertToLogFactTbl( String timeID, int userID,
                                  int protocolID, int pathID, int b ) {
      int count = -1;

      DataCollectorControl con = new DataCollectorControl();
      count = con.insertToLogFactTbl( timeID, userID, protocolID, pathID, b );

      return count;
   }
} // TFDatCollector
