/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import java.io.*;
import java.util.*;
import java.sql.*;

/**
  * This program will:
  *    1) create all tables of the database warehouse
  *    2) delete all tables of the database warehouse
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class TableModel {
   private Connection con = null;    // represent connection with the database

   /**
     * The constructor will establish connection with the database
     * specified in the configaration file 'configFile.INI'
     *
     * @exception   ClassNotFoundException         if class specified by driver cannot be located
     *
     * @exception   SQLException                   for sql exception
     */

   public TableModel () {
      // the singleton object which will store driver and connection names for database access
      Config config = Config.Factory();
      // getting driver
      String driver = config.getDriver();
      // getting connection
      String connection = config.getConnection();

      // the singleton object which will store user id and password for database access
      DBLogin login = DBLogin.Factory();
      // getting user id
      String userID = login.getUserID();
      // getting password
      String password = login.getPassword();

      try {
         // getting reference
         Class.forName ( driver );
         // getting connection
         con = DriverManager.getConnection( connection, userID, password );
      } catch ( ClassNotFoundException cnf ) {
         cnf.printStackTrace();
      } catch ( SQLException sqle ) {
         sqle.printStackTrace();
      }
   }

   /**
     * Method closeConnection will disconnect the program from the database
     * specified in the configaration file 'configFile.INI'
     *
     * @exception   SQLException                   for sql exception
     */

   public void closeConnection() {
      try {
         con.close();
      } catch ( SQLException sqle ) {
         sqle.printStackTrace();
      }
   }

   /**
     * Method createWarehouseTables will create four tables of the warehouse in the database
     *
     * @exception   SQLException            for sql exception
     *
     * @return                              0 if tables are created, or -1 otherwise
     */

   public int createWarehouseTables() {
     int count = 0;
     String str = "";
     Statement stmt = null;
     try {
        stmt = con.createStatement();
        str = "create table UserDimTbl ( "
            + "userID int primary key, "
            + "ipAddress varchar(20) not null, "
            + "hostName varchar(200), "
            + "userName varchar(200) )";
        stmt.executeUpdate( str );
        str = "create table ProtocolDimTbl ( "
            + "protocolID int primary key, "
            + "protocolName varchar(10) not null, "
            + "methodName varchar(15), "
            + "status int )";
        stmt.executeUpdate( str );
        str = "create table PathDimTbl ( "
            + "pathID int primary key, "
            + "pathName varchar(200), "
            + "fileName varchar(100), "
            + "fileType varchar(50) )";
        stmt.executeUpdate( str );
        str = "create table LogFactTbl ( "
            + "timeID varchar(30) not null, "
            + "userID int not null, "
            + "protocolID int not null, "
            + "pathID int not null, "
            + "sessionID int, "
            + "nbyte int, "
            + "CONSTRAINT protocolID_fk FOREIGN KEY ( protocolID ) REFERENCES ProtocolDimTbl ( protocolID ), "
            + "CONSTRAINT userID_fk FOREIGN KEY ( userID ) REFERENCES UserDimTbl ( userID ), "
            + "CONSTRAINT pathID_fk FOREIGN KEY ( pathID ) REFERENCES PathDimTbl ( pathID ) )";
        stmt.executeUpdate( str );
        stmt.close();
     } catch ( SQLException sqle ) {
        sqle.printStackTrace();
        count = -1;
     }
     return count;
   }

   /**
     * Method deleteWarehouseTables will delete four tables of the warehouse in the database
     *
     * @exception   SQLException            for sql exception
     *
     * @return                              0 if tables are deleted, or
     *                                      -1 otherwise
     */

   public int deleteWarehouseTables() {
     int count = 0;
     String str = "";
     Statement stmt = null;
     try {
        stmt = con.createStatement();
        str = "drop table LogFactTbl";
        stmt.executeUpdate( str );
        str = "drop table UserDimTbl";
        stmt.executeUpdate( str );
        str = "drop table ProtocolDimTbl";
        stmt.executeUpdate( str );
        str = "drop table PathDimTbl";
        stmt.executeUpdate( str );
        stmt.close();
     } catch ( SQLException sqle ) {
        sqle.printStackTrace();
        count = -1;
     }
     return count;
   }
} // TableModel
