/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import java.io.*;
import java.util.*;

/**
  * This class will hold values for:
  *    1) os
  *    2) copy command
  *    3) web access log file
  *    4) target file in input directory
  *
  *    5) database driver name
  *    6) database connection
  *
  *    7) sleep time
  *    8) next iteration
  *
  *    9) file path
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class Config {
   private static Config config;                // referenc to the singleton object

   private static String os;                    // will store operating system
   private static String cp;                    // will store copy command
   private static String serverAccessLogFile;   // will store web server access log file
   private static String accessLogFile;         // will store log file in input directory

   private static String driver;                // will store DB driver name
   private static String connection;            // will store DB connection name

   private static int sleepingTime;             // will store sleep time
   private static String nextIteration;         // will store yes or no for next iteration

   private static String path;                  // will store file path

   /**
     * The private constructor will initiate variables
     */

   private Config(){
      // initialise
      serverAccessLogFile = "";
      accessLogFile = "";
      cp = "";
      driver = "oracle.jdbc.driver.OracleDriver";
      connection = "jdbc:oracle:thin:@goedel.newcs.uwindsor.ca:1521:CS01";
      sleepingTime = 0;
      nextIteration = "";
      path = "";
      os = "";
      // method call to read configaration setting from the configaration file 'configFile.INI'
      getConfiguration();
   }

   /**
     * Method Factory will create a single object of the class
     */

   public static Config Factory() {
      if( config == null ) {
        config = new Config();
      }
      return config;
   }

   /**
     * Method getConfigaration will read configuration data
     * from config file 'configFile.INI' and assign appropriate values
     * to the corresponding variables
     *
     * @exception      FileNotFoundException     if file can not be located
     *
     * @exception      IOException               for io exception
     */

   private void getConfiguration() {
      int index = -1;

      String str = "";
      String temp = "";
      String hold = "";

      StringTokenizer tok;     // store tokens
      BufferedReader reader = null;

      try {
         reader = new BufferedReader( new FileReader(
                                         "../config/configFile.INI" ) );

         str = reader.readLine();
         // while loop to read the configaration file and assign appropriate values to the variables
         while( str != null ) {
            // disregard whatever comes after double slash
            index = str.indexOf( "//" );
            if( index != -1 ) {
               temp = str.substring( 0, index );
               str = temp;
            }
            // tokenization
            tok = new StringTokenizer( str, "= " );
            if( tok.hasMoreTokens() ) {
               // getting the 'name' part of the 'name=value' pair
               hold = tok.nextToken();
               //if-else selection structure to get corresponding 'value' for the 'name'
               if( hold.equals( "path" ) ) {
                  path = tok.nextToken();
               } else if( hold.equals( "serverAccessLogFile" ) ) {
                  serverAccessLogFile = tok.nextToken();
               } else if( hold.equals( "accessLogFile" ) ) {
                  accessLogFile = tok.nextToken();
               } else if( hold.equals( "copyCommand" ) ) {
                  cp = tok.nextToken();
               } else if( hold.equals( "driver" ) ) {
                  driver = tok.nextToken();
               } else if( hold.equals( "connection" ) ) {
                  connection = tok.nextToken();
               } else if( hold.equals( "sleepingTime" ) ) {
                  sleepingTime = Integer.parseInt( tok.nextToken() );
               } else if( hold.equals( "nextIteration" ) ) {
                  nextIteration = tok.nextToken();
               } else if( hold.equals( "os" ) ) {
                  os = tok.nextToken();
               }
            }
            str = reader.readLine();
         }
         reader.close();
      } catch( FileNotFoundException fnf ) {
        System.out.println("Cannot locate input file! " + fnf.getMessage() );
      } catch( IOException ioe ){
        System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method doNextIteration will enable the user to come out of infinite loop
     * by changing configuration file.
     *
     * @return                       true if next iteration is required, false otherwise
     */

   public boolean doNextIteration() {
      boolean flag = false;

      // re-initialized the variables
      getConfiguration();
      if( nextIteration != null && nextIteration.equals( "yes" ) ) {
         flag = true;
      }

      return flag;
   }

   /**
     * getter methods
     */
   public String getServerAccessLogFile() {
      return serverAccessLogFile;
   }
   public String getAccessLogFile() {
      return accessLogFile;
   }
   public String getCopyCommand() {
      return cp;
   }
   public String getDriver() {
      return driver;
   }
   public String getConnection() {
      return connection;
   }
   public String getPath() {
       return path;
   }
   public int getSleepingTime() {
      return sleepingTime;
   }
   public String getOs() {
       return os;
   }
} // Config
