/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import javax.swing.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
  * This program will:
  *    1) get an access record as string
  *    2) separete data entries
  *    3) clean the data by remove unwantted characters
  *    4) check if any part of the record is missing, if no part is missing, populate ProcssData
  *    5) check if the record contains invalid file type
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class DataManager {
   private boolean flag;             // indicate data validity
   private String path;              // file path of invalidFileType file
   private String rawData;           // stores access record as string
   private String[] processedData;   // will store cleaned data of the record
   private FilePath fp;              // to find file path, file and file type of the record

   /**
     * The constructor will initialized variables.
     * It will call processData method to clean data of the record
     * and store data entries as seperate string
     *
     * @param     data           raw data of a record as string
     */

   public DataManager( String data ) {
      flag = false;
      fp = null;

      // the singleton object which will store file path
      Config config = Config.Factory();
      // getting file path
      path = config.getPath();

      processedData = new String[12];
      rawData = data;
      // for clean data
      processData();
   }

   /**
     * Method setRawData will initialized flag and rawData variables.
     * It will call processData method to clean data of the record
     * and store data entries as seperate string
     */

   public void setRawData( String data ) {
      flag = false;
      rawData = data;
      // for clean data
      processData();
   }

   /**
     * getter method
     */

   public String[] getProcessedData() {
      return processedData;
   }

   /**
     * helper method
     */

   public boolean isValidData() {
      return flag;
   }

   /**
     * Method processData will
     *    1) call findData to separete data entries
     *    2) call cleanData to remove unwantted characters
     *    3) check if any part of the record is missing
     *    4) if no part is missing, populate ProcssData
     *
     * @exception   FileNotFoundException     if file cannot be located
     *
     * @exception   IOException               for io exceptions
     */

   public void processData() {
      // to seperate data entries
      String[] temp = findData( rawData );
      // cleaning
      //IP address is stored in arg[0] );
      //Host Name is stored in arg[1] );
      //User is stored in arg[2] );
      //Timestamp is stored in  arg[3] );
      //Offset is stored in arg[4] );
      //Method is stored in arg[5] );
      //Path & File is stored in arg[6] );
      //Protocol is stored in arg[7] );
      //Status is stored in arg[8] );
      //Bytes is stored in arg[9] );
      String[] arg = cleanData( temp );
      boolean flag1 = true;
      // checking missing section
      for( int i = 0; i < arg.length; i++ ) {
         // if the record has missing section,
         // it will be ignored
         if( arg[i] == null ) {
            flag1 = false;
            break;
         }
      }
      if( flag1 ) {
         //IP address is stored in processedData[0] );
         processedData[0] = arg[0];
         //Host Name is stored in processedData[1] );
         processedData[1] = arg[1];
         //User is stored in processedData[2] );
         processedData[2] = arg[2];
         //Timestamp is stored in  processedData[3] );
         processedData[3] = arg[3];
         //offset is stored in processedData[4] );
         processedData[4] = arg[4];
         //Method is stored in processedData[5] );
         processedData[5] = arg[5];

         // to get Path, file and file type
         if( fp == null ) {
           fp = new FilePath( arg[6] );
         } else {
           fp.setFilePath( arg[6] );
         }
         String[] hold = fp.getPathFileType();
         //Path is stored in processedData[6] );
         processedData[6] = hold[0];
         //File is stored in processedData[7] );
         processedData[7] = hold[1];
         //File Type is stored in processedData[8] );
         processedData[8] = hold[2];

         // if structure to find valid path identifier
         boolean f1 = true;
         for( int i = 0; i < processedData[6].length(); i++ ) {
            char ch = processedData[6].charAt( i );
            if( i == 0 && !Character.isJavaIdentifierStart( ch )
                && !( ch == '/' ) && !( ch == '~' ) && !( ch == '%' ) ) {
                f1 = false;
                break;
            } else if( i == 1 && !Character.isJavaIdentifierPart( ch )
                       && !( ch == '/' ) && !( ch == '~' ) && !( ch == '%' ) ) {
                f1 = false;
                break;
            } else if( i > 1 && !Character.isJavaIdentifierPart( ch )
                       && !( ch == '/' ) && !( ch == '~' ) && !( ch == '-' ) && !( ch == '%' ) ) {
                f1 = false;
                break;
            }
         }
         // if structure to find valid file name identifier
         boolean f2 = true;
         for( int i = 0; i < processedData[7].length(); i++ ) {
            char ch = processedData[7].charAt( i );
            if( i == 0 && !Character.isJavaIdentifierStart( ch ) && !( ch == '%' ) ) {
                f1 = false;
                break;
            } else if( i > 0 && !Character.isJavaIdentifierPart( ch )
                       && !( ch == '-' ) && !( ch == '%' ) && !( ch == '.' )) {
                f1 = false;
                break;
            }
         }
         // if structure to find valid file type identifier
         boolean f3 = true;
         for( int i = 0; i < processedData[8].length(); i++ ) {
            char ch = processedData[8].charAt( i );
            if( i == 0 && !Character.isJavaIdentifierStart( ch ) && !( ch == '%' ) ) {
                f1 = false;
                break;
            } else if( i > 0 && !Character.isJavaIdentifierPart( ch )
                       && !( ch == '-' ) && !( ch == '%' ) ) {
                f1 = false;
                break;
            }
         }
         // if structure to find only header file access
         boolean f4 = true;
         if( processedData[6].equals( "/" ) && processedData[7].equals( "-" ) && processedData[8].equals( "-" ) ) {
            f4 = false;
         }
         // if structure to check file type
         if( f1 && f2 && f3 && f4 ) {
            // checking file type
            flag = isValidFileType( processedData[8] );
         }
         // if structure to keep significant portions of the identifiers
         if( flag ) {
            // removing front portion of the path name to make it of 200 charecters
            int i = -1;
            String p = processedData[6];
            i = p.length();
            if( i > 200 ) {
               p = p.substring( i - ( 200 - 1 ) );
               p = "=" + p;
               processedData[6] = p;
            }
            // truncating the file name to 100 charecters
            i = -1;
            String f = processedData[7];
            i = f.length();
            if( i > 100 ) {
               f = f.substring( 0, 100 );
               processedData[7] = f;
            }
            // truncating the file extension to 50 charecters
            i = -1;
            String t = processedData[8];
            i = t.length();
            if( i > 50 ) {
               t = t.substring( 0, 50 );
               processedData[8] = t;
            }
         }
         //Protocol is stored in processedData[9] );
         processedData[9] = arg[7];
         //Status is stored in processedData[10] );
         processedData[10] = arg[8];
         //number of Bytes is stored in processedData[11] );
         processedData[11] = arg[9];
      }
    }

   /**
     * Method findData seperates entries of the record
     *
     * @param       str      str is a String
     *
     * @return               array of Strings
     */

   public String[] findData( String str ) {
      String[] hold;           // stores entries of the record
      StringTokenizer tok;     // store tokens

      hold = new String[10];

      if ( str != null ) {
         // tokenizing
         tok = new StringTokenizer( str, " " );

         int i = 0;
         // while loop to assign entries of the record to the array hold
         while( tok.hasMoreTokens() && i < 10 ) {
            hold[i] = tok.nextToken();
            i++;
         }
      }
      return hold;
   }

   /**
     * Method cleanData cleans entries of the record
     *
     * @param       str      str is a String
     *
     * @return               array of Strings
     */

   public String[] cleanData( String[] str ) {
      int index;
      String temp = "";
      String[] hold;
      StringTokenizer tok;

      hold = new String[10];

      // ip
      if( str[0] != null ) {
         hold[0] = str[0];
      }
      // host name
      if( str[1] != null ) {
         hold[1] = str[1];
      }
      // user
      if( str[2] != null ) {
         hold[2] = str[2];
      }
      // raw TimeStamp
      if( str[3] != null ) {
         //removing '['
         index = str[3].indexOf( "[" );
         if( index != -1 ) {
            index++;
            temp = str[3].substring( index, str[3].length() );
            hold[3] = temp;
         } else {
            hold[3] = str[3];
         }
      }
      // raw offset
      if( str[4] != null ) {
         //removing ']'
         index = str[4].indexOf( "]" );
         if( index != -1 ) {
            hold[4] = str[4].substring( 0, index );
         } else {
            hold[4] = str[4];
         }
      }
      // raw Method
      if( str[5] != null ) {
         // removing \"
         index = str[5].indexOf( "\"" );
         if( index != -1 ) {
            index++;
            hold[5] = str[5].substring( index, str[5].length() );
         } else {
            hold[5] = str[5];
         }
      }
      // raw path & file
      // removing portion of the string after ?
      if( str[6] != null ) {
         index = str[6].indexOf( "?" );
         if( index >= 0 ) {
            temp = str[6].substring( 0, index );
            temp += "?";
            str[6] = temp;
         }
         hold[6] = str[6];
      }
      // raw protocol
      if( str[7] != null ) {
         // removing \"
         index = str[7].indexOf( "\"" );
         if( index != -1 ) {
            hold[7] = str[7].substring( 0, index );
         } else {
            hold[7] = str[7];
         }
      }
      // status
      if( str[8] != null ) {
        hold[8] = str[8];
      }
      // no of bytes
      if( str[9] != null ) {
        hold[9] = str[9];
      }
      return hold;
   }

   /**
     * Method isValidFileType reads
     * the file listing invalid file type line by line
     * and compare argument st with the invalid file entries.
     * If there is a match, it return false.
     * Otherwise it return true.
     *
     * @param       st                      st is a String
     *
     * @exception   FileNotFoundException   if file cannot be located
     *
     * @exception   IOException             for io exceptions
     *
     * @return                              boolean
     */

   public boolean isValidFileType( String st ) {
      boolean flag = true;

      if( st == null ) {
         flag = false;
         return flag;
      } else if( st.equals( "-" ) ) {
         flag = true;
         return flag;
      } else {
         BufferedReader reader = null;
         // converting to lower case
         String temp = st.toLowerCase();

         String hold = null;
         String str = null;
         try {
            reader = new BufferedReader(
                        new FileReader(
                            path + "invalidFileType/invalidFileType" ) );

            str = reader.readLine();
            str = reader.readLine();
            // while loop to read invalide file type entry
            // and compare it with the lower case version of the argument
            while( str != null ) {
               // tokenizing
               StringTokenizer tok = new StringTokenizer( str, " ." );

               if( tok.hasMoreTokens() ) {
                  hold = tok.nextToken();
                  // comparing with temp the lower case vertion of the argument st
                  if( temp.equals( hold ) ) {
                     flag = false;
                  }
               }
               // if match is found
               if( !flag ) {
                 break;
               }
               str = reader.readLine();
            }
            reader.close();

         } catch( FileNotFoundException fnf ) {
            System.out.println("DataManger: isValidFileType: Cannot locate input file! " + fnf.getMessage() );
         } catch( IOException ioe ){
            System.out.println( ioe.getMessage() );
         }
      }
      return flag;
   }
} // DataManager
