/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import javax.swing.*;
import java.io.*;
import java.util.*;

/**
  * This program will create the sequence of web pages visited
  * by a user or users in a text file.
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class DataSetSequence {
   private ArrayList al;             // stores access record
   private String path;              // file path
   private String fileName;          // data set file name
   private String footNote;          // store note string for the data sets


   /**
    * The constructor will initialize variables
    */

   public DataSetSequence( ArrayList l, String fName, String fn ) {
      al = l;
      fileName = fName;
      footNote = fn;
      // the singleton object which will store file path
      Config config = Config.Factory();
      // getting file path
      path = config.getPath();
   }

   /**
     * Method buildWebpageVisitSeq will create a file which will list userID,
     * sessionID, and sequence of pathID visited by the users in the same session.
     *
     * @exception   IOException              for io exception
     */

   public void buildWebpageVisitSeq() {
      int size;                  // number of record
      int max;                   // store highest userID or sessionID

      size = al.size();
      // populating MyRecord array
      MyRecord[] A1 = new MyRecord[ size ];
      for( int i = 0; i < size; i++ ) {
         A1[i] = ( MyRecord )al.get( i );
      }
      // getting highest sessionID
      MyRecord hold = ( MyRecord ) al.get( 0 );
      max = hold.getSessionID();
      for( int i = 0; i < size; i++ ) {
         MyRecord temp = ( MyRecord )al.get( i );
         if(  temp.getSessionID() > max ) {
            max = temp.getSessionID();
            hold = temp;
         }
      }
      max++;
      // method call to sort by sessionID
      MyRecord[] B = sortBySession( A1, size, max );

      // populating MyRecord array with the MyRecords sorted by sessionID
      MyRecord[] A2 = new MyRecord[ size ];
      for( int i = 1; i <= size; i++ ) {
         A2[i - 1] = B[i];
      }
      // getting highest userID
      hold = ( MyRecord ) al.get( 0 );
      max = hold.getUserID();
      for( int i = 0; i < size; i++ ) {
         MyRecord temp = ( MyRecord )al.get( i );
         if(  temp.compareTo( hold ) >= 1 ) {
            max = temp.getUserID();
            hold = temp;
         }
      }
      max++;
      // method call to sort by userID
      B = sortByUser( A2, size, max );

      // a space string
      String sp = " ";
      while( sp.length() < 22 ) {
         sp += " ";
      }
      // header string
      String s = "userID ";
      while( s.length() < 10 ) {
         s += " ";
      }
      s += "sessionID ";
      while( s.length() < 22 ) {
         s += " ";
      }
      s += "web page visited\n";

      PrintWriter pw = null;
      try {
         pw = new PrintWriter (
                   new BufferedWriter (
                       new FileWriter ( path + "dataset/" + fileName + ".suv" ) ) );
         // writing header of the file
         pw.print( s );
         // entry for a user
         int userID = B[1].getUserID();
         s = userID + " ";
         while( s.length() < 10 ) {
            s += " ";
         }
         int sessionID = B[1].getSessionID();
         s += sessionID + " ";
         while( s.length() < 22 ) {
            s += " ";
         }
         for( int i = 1; i < B.length; i++ ) {
            // finding next user id with session id
            int uID = B[i].getUserID();
            int sID = B[i].getSessionID();
            // if-else selection to check userID and sessionID of two adjacent entries
            if( uID == userID && sID == sessionID ) {
               // for same userID and sessionID

               // if selection to write the string
               if( s.length() >= 60 ) {
                  s += "\n";
                  pw.print( s );
                  // starting a new string
                  s = sp;
               }
               // getting pathID
               s += B[i].getPathID() + " ";
            } else {
               // for different userID or sessionID

               // wrting the existing string
               s += "\n";
               pw.print( s );
               // starting a new string
               s = uID + " ";
               while( s.length() < 10 ) {
                  s += " ";
               }
               s += sID + " ";
               while( s.length() < 22 ) {
                  s += " ";
               }
               // getting pathID
               s += B[i].getPathID() + " ";
               userID = uID;
               sessionID = sID;
            }
         }
         // writing
         s += "\n";
         pw.print( s );
         pw.println( footNote );
         pw.close();
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
   }

   /**
     * Method sortByUser will sort the array of the MyRecord based on userID
     * using counting sort technique
     *
     * @param                  A                an array of MyRecord
     *
     * @param                  size             size of the array of MyRecord
     *
     * @param                  max              highest value of the userID
     *
     * @return                                  an array of MyRecord
     */

   public MyRecord[] sortByUser( MyRecord[] A, int size, int max ) {
     // creating and initializing an int array C
     int[] C = new int[ max ];
     for( int i = 0; i < max ; i++ ) {
        C[i] = 0;
     }
     // C stores number of userID in its cell number equals to the userID
     for( int i = 0; i < size; i++ ) {
        C[ A[i].getUserID() ] = C[ A[i].getUserID() ] + 1;
     }
     // C stores how many userIds are equals to or less than cell number
     for( int i = 1; i < max; i++ ) {
        C[ i ] = C[ i - 1 ] + C[ i ];
     }
     // B will a sorted array
     MyRecord[] B = new MyRecord[ size + 1 ];
     // to get a sorted array
     for( int n = size - 1; n >= 0; n-- ) {
        B[ C[ A[n].getUserID() ] ] = A[n];
        C[ A[n].getUserID() ] = C[ A[n].getUserID() ] - 1;
     }
     return B;
   }

   /**
     * Method sortBySession will sort the array of the MyRecord based on sessionID
     * using counting sort technique
     *
     * @param                  A                an array of MyRecord
     *
     * @param                  size             size of the array of MyRecord
     *
     * @param                  max              highest value of the userID
     *
     * @return                                  an array of MyRecord
     */

   public MyRecord[] sortBySession( MyRecord[] A, int size, int max ) {
     // creating and initializing an int array C
     int[] C = new int[ max ];
     for( int i = 0; i < max ; i++ ) {
        C[i] = 0;
     }
     // C stores number of sessionID in its cell number equals to the sessionID
     for( int i = 0; i < size; i++ ) {
        C[ A[i].getSessionID() ] = C[ A[i].getSessionID() ] + 1;
     }
     // C stores how many sessionIds are equals to or less than cell number
     for( int i = 1; i < max; i++ ) {
        C[ i ] = C[ i - 1 ] + C[ i ];
     }
     // B will a sorted array
     MyRecord[] B = new MyRecord[ size + 1 ];
     // to get a sorted array
     for( int n = size - 1; n >= 0; n-- ) {
        B[ C[ A[n].getSessionID() ] ] = A[n];
        C[ A[n].getSessionID() ] = C[ A[n].getSessionID() ] - 1;
     }
     return B;
   }
} // DataSetSequence
