/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.sql.*;

/**
  * This program will:
  *    1) get host name corresponding to ip address from DNS
  *    2) update the UserDimTbl with the host name
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class HostControl {
   private String path;

   /**
     * The constructor will get file path name from singleton Config object
     */

   public HostControl() {
      // the singleton object which will store file path
      Config config = Config.Factory();
      // getting file path
      path = config.getPath();
   }

   /**
     * Method updateTFUserDimTbl will check host name of UserDimTble.
     * If host name is not found, it will get host name corresponding ip address from DNS
     * and update the table of the warehouse in the text files
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException             io exception
     *
     * @return                              number of record updated
     */

   public int updateTFUserDimTbl() {
      int count = 0;                  // number of host name updates
      int uID = -1;                   // user id of the current record
      String hold = "";               // user id of the current record as String
      String ip = "";                 // IP address of the current record
      String host = "";               // host name corresponding to IP address of the current record
      String user = "";               // user name corresponding to IP address of the current record

      PrintWriter pw = null;          // writer
      String str = "";                // current record
      BufferedReader reader = null;   // reader

      try {
         // getting reference to create a temporary file for UserDimTbl
         pw = new PrintWriter( new BufferedWriter( new FileWriter ( path + "output/temp"  ) ) );
         // creating header of temp file
         String s = "userID ";
         while( s.length() < 15 ) {
            s += " ";
         }
         s += "ipAddress ";
         while( s.length() < 35 ) {
            s += " ";
         }
         s += "hostName ";
         while( s.length() < 100 ) {
            s += " ";
         }
         s += "userName";
         // writing header to temp file
         pw.println( s );

         // getting reference to read UserDimTbl
         reader = new BufferedReader( new FileReader( path + "output/UserDimTbl" ) );
         str = reader.readLine();
         str = reader.readLine();
         // while loop to check all the entries of the UserDimTble
         // if they have host names corresponding to IP address
         while( str != null ) {
            // tokenizing the current record
            StringTokenizer tok = new StringTokenizer( str, " \t" );

            // getting component of current record
            if( tok.hasMoreTokens() ) {
               hold = tok.nextToken();
               uID = Integer.parseInt( hold );
            }
            if( tok.hasMoreTokens() ) {
               ip = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               host = tok.nextToken();
            }
            if( tok.hasMoreTokens() ) {
               user = tok.nextToken();
            }

            // if a record has no host name
            if( host.equals( "-" ) ) {
               // getting host name corresponding to the IP address
               String hostName = getHostName( ip );
               // update string
               String update = uID + " ";
               while( update.length() < 15 ) {
                  update += " ";
               }
               update += ip + " ";
               while( update.length() < 35 ) {
                  update += " ";
               }
               update += hostName + " ";
               while( update.length() < 100 ) {
                  update += " ";
               }
               update += user;
               // writing update to temp file
                pw.println( update );
                count++;
            } else {
               pw.println( str );
            }
            if( count >= 1 && count % 100 == 0 ) {
               System.out.println( "Till now " + count + " host names are updated..." );
            }
            str = reader.readLine();
         }
         reader.close();
         pw.close();

         // deleting UserDimTbl
         boolean flag = deleteUserDimTbl();
         // renameing temp file as UserDimTbl
         if( flag ) {
            renameTempFile();
         }
       } catch( FileNotFoundException fnf ) {
          System.out.println("Cannot locate input file! " + fnf.getMessage() );
       } catch( IOException ioe ){
          System.out.println( ioe.getMessage() );
       }
       return count;
    }

   /**
     * Method getHostName will access DNS for host name correseponding to the IPAddress
     *
     * @param       ipAddrs                   IP Address
     *
     * @exception   UnknownHostException      if host is not known
     *
     * @exception   IOException               for io exceptions
     *
     * @return                                Host name
     */

   public String getHostName( String ipAddrs ) {
      String host = "-";
      try {
         // getting reference to InetAddress object
         InetAddress ia = InetAddress.getByName( ipAddrs );
         // method call
         host = ia.getHostName();
      } catch( UnknownHostException uh ) {
         System.out.println("HostControl: Cannot locate host! " + uh.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return host;
   }

   /**
     * Method deleteUserDimTbl will delete existing UserDimTbl
     *
     * @return                                true or false
     */

   public boolean deleteUserDimTbl() {
         File udt;
         boolean flag = false;
         // creating a File object
         udt = new File( path + "output/UserDimTbl" );
         if( udt.exists() ) {
            // delete UserDimTbl file if the file exists
            flag = udt.delete();
         }
         return flag;
   }

   /**
     * Method renameTempFile will rename temp file as UserDimTbl
     */

   public void renameTempFile() {
      // creatin File objects
      File nudt = new File( path + "output/UserDimTbl" );
      File tudt = new File( path + "output/temp" );
      // renaming
      if( tudt.exists() ) {
         tudt.renameTo( nudt );
      }
   }
} // HostControl
