/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import java.io.*;
import java.util.*;
import java.sql.*;

/**
  * This program will:
  *    1) create all files of the text files warehouse in output directory
  *    2) delete all files of the text files warehouse from output directory
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class TableControl {
   private String path;

   /**
     * The constructor will get file path name from singleton Config object
     */

   public TableControl() {
      // the singleton object which will store file path
      Config config = Config.Factory();
      // getting file path
      path = config.getPath();
   }

  /**
     * Method createWarehouseTables will create four text files
     * which will be used as tables of the text files warehouse
     *
     * @exception   FileNotFoundException   if input file can not be located
     *
     * @exception   IOException             io exception
     *
     * @return                              0 if tables are created, or -1 otherwise
     */

   public int createWarehouseTables() {
      int count = 0;
      String s;
      File udt;
      File protdt;
      File padt;
      File lft;
      PrintWriter pw;

      try {
         // creating UserDimTbl file
         udt = new File( path + "output/UserDimTbl" );
         // return -1 if the file exists, create the file otherwise
         if( udt.exists() ) {
            count = -1;
            return count;
         } else {
            count = -1;
            pw = new PrintWriter (
                      new BufferedWriter (
                          new FileWriter( udt ) ) );
            // creating header of UseDimTbl
            s = "";
            s += "userID ";
            while( s.length() < 15 ) {
               s += " ";
            }
            s += "ipAddress ";
            while( s.length() < 35 ) {
               s += " ";
            }
            s += "hostName ";
            while( s.length() < 100 ) {
               s += " ";
            }
            s += "userName";
            // writing header to UserDimTbl file
            pw.println( s );
            pw.close();
            count = 0;
         }

         // creating ProtocolDimTbl file
         protdt = new File( path + "output/ProtocolDimTbl" );
         // return -1 if the file exists, create the file otherwise
         if (protdt.exists()) {
            count = -1;
            return count;
         } else {
            count = -1;
            pw = new PrintWriter (
                      new BufferedWriter (
                          new FileWriter( protdt ) ) );
            // creating header of ProtocolDimTbl
            s = "";
            s += "protocolID ";
            while( s.length() < 15 ) {
               s += " ";
            }
            s += "protocolName ";
            while( s.length() < 35 ) {
               s += " ";
            }
            s += "methodName ";
            while( s.length() < 50 ) {
               s += " ";
            }
            s += "status";
            // writing header to ProtocolDimTbl file
            pw.println( s );
            pw.close();
            count = 0;
         }

         // creating ProtocolDimTbl file
         padt = new File( path + "output/PathDimTbl" );
         // return -1 if the file exists, create the file otherwise
         if (padt.exists()) {
            count = -1;
            return count;
         } else {
            count = -1;
            pw = new PrintWriter (
                      new BufferedWriter (
                          new FileWriter( padt ) ) );
            // creating header of PathDimTbl
            s = "";
            s += "pathID ";
            while( s.length() < 15 ) {
               s += " ";
            }
            s += "pathName ";
            while( s.length() < 75 ) {
               s += " ";
            }
            s += "fileName ";
            while( s.length() < 120 ) {
               s += " ";
            }
            s += "fileType";
            // writing header to PathDimTbl file
            pw.println( s );
            pw.close();
            count = 0;
         }

         // creating LogFactTbl file
         lft = new File( path + "output/LogFactTbl" );
         // return -1 if the file exists, create the file otherwise
         if (lft.exists()) {
            count = -1;
            return count;
         } else {
            count = -1;
            pw = new PrintWriter (
                      new BufferedWriter (
                          new FileWriter( lft ) ) );
            // creating header of PathDimTbl
            s = "";
            s += "timeID ";
            while( s.length() < 30 ) {
               s += " ";
            }
            s += "userID ";
            while( s.length() < 40 ) {
               s += " ";
            }
            s += "protocolID ";
            while( s.length() < 55 ) {
               s += " ";
            }
            s += "pathID ";
            while( s.length() < 65 ) {
               s += " ";
            }
            s += "sessionID ";
            while( s.length() < 80 ) {
               s += " ";
            }
            s += "nbyte";
            // writing header to LogFactTbl file
            pw.println( s );
            pw.close();
            count = 0;
         }
      } catch( FileNotFoundException fnf ) {
         System.out.println("Control: Cannot locate output file! " + fnf.getMessage() );
      } catch( IOException ioe ){
         System.out.println( ioe.getMessage() );
      }
      return count;
   }

   /**
     * Method deleteWarehouseTables will delete four text files
     * which are used as tables of the warehouse
     *
     * @return                      true if tables are deleted, or false otherwise
     */

   public boolean deleteWarehouseTables() {
      boolean flag = false;
      File udt;
      File protdt;
      File padt;
      File lft;
      PrintWriter pw;

      // creating a File object
      udt = new File( path + "output/UserDimTbl" );
      if( udt.exists() ) {
         // delete UserDimTbl file if the file exists
         flag = udt.delete();
         if( !flag ) {
            return flag;
         }
      }
      flag = false;
      // creating a File object
      protdt = new File( path + "output/ProtocolDimTbl" );
      if (protdt.exists()) {
         // delete ProtocolDimTbl file if the file exists
         flag = protdt.delete();
         if( !flag ) {
            return flag;
         }
      }
      flag = false;
      // creating a File object
      padt = new File( path + "output/PathDimTbl" );
      if (padt.exists()) {
         // delete PathDimTbl file if the file exists
         flag = padt.delete();
         if( !flag ) {
            return flag;
         }
      }
      flag = false;
      // creating a File object
      lft = new File( path + "output/LogFactTbl" );
      if (lft.exists()) {
         // delete LogFactTbl file if the file exists
         flag = lft.delete();
      }
      return flag;
   }
} // TableControl
