/*
 * Version:
 *    $Research Project of Undergraduate Student Research Award$
 *    $Tools for Mining and Building Data Warehouses$
 *
 * Revisions:
 *   $Date : August 20, 2004$
 *
 *   $Date : January 10, 2005$
 */

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.awt.event.*;

/**
  * This class is a top level program which will provide user interface:
  *    1) to collect data from web access log file
  *       and store the cleaned data to either databse warehouse,
  *       or text file warehouse
  *    2) to get host name from DNS and to store it to warehouse
  *    3) to create warehouse either in database or in text file warehouse
  *    4) to delete warehouse either from database or from text file warehouse
  *    5) to generate data set based on time, user, protocol, or path information
  *       from either database warehouse, or text file warehouse  *
  *
  * @author   MOHAMMED LIAKAT ALI   Student
  *                                 School of Computer Science
  *                                 University of Windsor
  *
  * @supervisor Dr. CHRISTIE EZEIFE Associate Professor
  *                                 School of Computer Science
  *                                 University of Windsor
  */

public class WebAccessLogMonitor extends JFrame implements ActionListener {
   private Container c;        // a container
   private JTextArea out;      // will display message of execution
   private int index;          // will identify user selection
   private Config config;      // will store configaration information
   private Selector selector;  // will store selection information

   private JMenuItem stdb;     // will initiate storing to database warehouse
   private JMenuItem sttf;     // will initiate storing to text file warehouse

   private JMenuItem gtdb;     // will initiate getting and storing host name to database warehouse
   private JMenuItem gttf;     // will initiate getting and storing host name to text file warehouse

   private JMenuItem cidb;     // will initiate building database warehouse
   private JMenuItem citf;     // will initiate building text file warehouse

   private JMenuItem dfdb;     // will initiate deleting database warehouse
   private JMenuItem dftf;     // will initiate deleting text file warehouse

   private JMenuItem t1;       // will initiate creating time based data set from database warehouse
   private JMenuItem u1;       // will initiate creating user based data set from database warehouse
   private JMenuItem prot1;    // will initiate creating protocol based data set from database warehouse
   private JMenuItem pa1;      // will initiate creating path based data set from database warehouse

   private JMenuItem tt1;      // will initiate creating time based data set from text file warehouse
   private JMenuItem ut1;      // will initiate creating user based data set from text file warehouse
   private JMenuItem prott1;   // will initiate creating protocol based data set from text file warehouse
   private JMenuItem pat1;     // will initiate creating path based data set from text file warehouse

   private JButton stwh;       // used to store data in warehouse
   private JButton gtwh;       // used to get host name and store it in warehouse
   private JButton cwht;       // used to create warehouse tables
   private JButton dwht;       // used to delete warehouse tables
   private JButton cds;        // used to create data set

   private boolean stwhAdded;  // will indicate if stwht is added or not with the container
   private boolean gtwhAdded;  // will indicate if gtwht is added or not with the container
   private boolean cwhtAdded;  // will indicate if cwht is added or not with the container
   private boolean dwhtAdded;  // will indicate if dwht is added or not with the container
   private boolean cdsAdded;   // will indicate if cds is added or not with the container

   private String[] names;     // will store data collection modes
   private JComboBox cb;       // for selection of data collection mode

  /**
    * The constructor will create the main GUI for user interface
    */

   public WebAccessLogMonitor() {
      super( "Web Access Log Monitor" );
      // getting reference of the Config singlton object
      config = Config.Factory();
      // getting reference of the Selection singlton object
      selector = Selector.Factory();
      // buttons are not added yet to the container
      stwhAdded = false;
      gtwhAdded = false;
      cwhtAdded = false;
      dwhtAdded = false;
      cdsAdded = false;
      // creating event handler objects
      StoreEventHandler seh = new StoreEventHandler();
      GetEventHandler geh = new GetEventHandler();
      CreateEventHandler ceh = new CreateEventHandler();
      DeleteEventHandler deh = new DeleteEventHandler();

      // 'File' menu has 'Store Data in', 'Update Host Name of', and 'Exit' sub menus
      JMenu fMenu = new JMenu( "File" );
      fMenu.setMnemonic( 'F');

      // 'Store Data to' has
      // 'Database Warehouse' and 'Text Files Warehouse' sub menus
      JMenu sMenu = new JMenu( "Store Data to" );
      sMenu.setMnemonic( 'S' );
      stdb = new JMenuItem( "Database Warehouse" );
      stdb.addActionListener( seh );
      sMenu.add( stdb );
      sttf = new JMenuItem( "Text Files Warehouse" );
      sttf.addActionListener( seh );
      sMenu.add( sttf );
      fMenu.add( sMenu );

      // 'Update Host Name in' has
      // 'Database Warehouse' and 'Text Files Warehouse' sub menus
      JMenu gMenu = new JMenu( "Update Host Name in" );
      gMenu.setMnemonic( 'S' );
      gtdb = new JMenuItem( "Database Warehouse" );
      gtdb.addActionListener( geh );
      gMenu.add( gtdb );
      gttf = new JMenuItem( "Text Files Warehouse" );
      gttf.addActionListener( geh );
      gMenu.add( gttf );
      fMenu.add( gMenu );

      // The program will exit at the selection of 'Exit' sub menu
      JMenuItem eItem = new JMenuItem( "Exit" );
      eItem.setMnemonic( 'x' );
      // adding ActionListener
      eItem.addActionListener(
         new ActionListener() {
            public void actionPerformed( ActionEvent et ) {
               System.exit( 0 );
            }
         }
      );
      fMenu.add( eItem );

      // 'edit' menu has 'Create', 'Delete',
      // and 'Create Data Set' as sub menu
      JMenu eMenu = new JMenu( "Edit" );
      eMenu.setMnemonic( 'd');

      // 'Create' has 'Database Warehouse' and 'Text Files Warehouse' sub menus
      JMenu cMenu = new JMenu( "Create" );
      cMenu.setMnemonic( 'C' );
      cidb = new JMenuItem( "Database Warehouse" );
      cidb.addActionListener( ceh );
      cMenu.add( cidb );
      citf = new JMenuItem( "Text Files Warehouse" );
      citf.addActionListener( ceh );
      cMenu.add( citf );
      eMenu.add( cMenu );

      // 'Dalete' has 'Database Warehouse' and 'Text Files Warehouse' sub menus
      JMenu dMenu = new JMenu( "Delete" );
      dMenu.setMnemonic( 'L' );
      dfdb = new JMenuItem( "Database Warehouse" );
      dfdb.addActionListener( deh );
      dMenu.add( dfdb );
      dftf = new JMenuItem( "Text Files Warehouse" );
      dftf.addActionListener( deh );
      dMenu.add( dftf );
      eMenu.add( dMenu );

      // 'Create Data Set' has 'from Database Warehouse'
      // and 'from Text File Warehouse' as sub menu
      JMenu cdsMenu = new JMenu( "Create Data Set" );
      cdsMenu.setMnemonic( 'D' );
      // 'from Database Warehouse' has four sub menus
      JMenu tdb1 = new JMenu( "from Database Warehouse" );
      tdb1.setMnemonic( 'L' );
      cdsMenu.add( tdb1 );
      t1 = new JMenuItem( "based on time ID" );
      t1.addActionListener( this );
      tdb1.add( t1 );
      u1 = new JMenuItem( "based on user ID" );
      u1.addActionListener( this );
      tdb1.add( u1 );
      prot1 = new JMenuItem( "based on protocol ID" );
      prot1.addActionListener( this );
      tdb1.add( prot1 );
      pa1 = new JMenuItem( "based on path ID" );
      pa1.addActionListener( this );
      tdb1.add( pa1 );
      // 'from Text Files Warehouse' has four sub menus
      JMenu ttf1 = new JMenu( "from Text Files Warehouse" );
      cdsMenu.add( ttf1 );
      tt1 = new JMenuItem( "based on time ID" );
      tt1.addActionListener( this );
      ttf1.add( tt1 );
      ut1 = new JMenuItem( "based on user ID" );
      ut1.addActionListener( this );
      ttf1.add( ut1 );
      prott1 = new JMenuItem( "based on protocol ID" );
      prott1.addActionListener( this );
      ttf1.add( prott1 );
      pat1 = new JMenuItem( "based on path ID" );
      pat1.addActionListener( this );
      ttf1.add( pat1 );
      eMenu.add( cdsMenu );

      // adding 'file' and 'edit' submanus to menu bar
      JMenuBar b = new JMenuBar();
      b.add( fMenu );
      b.add( eMenu );
      setJMenuBar( b );

      // getting a container reference
      c = getContentPane();
      // setting Layout manager
      c.setLayout( new FlowLayout() );
      // setting background color
      c.setBackground( Color.CYAN );
      // creating an display object
      out = new JTextArea( 20, 56 );
      // initial prompt
      out.setText( "Please select from file menu then push store data button\n"
             + "1)To store data in database warehouse\n"
             + "2)To store data in text file warehouse\n\n"
             + "Please select from edit menu then push data set button\n"
             + "1)To create data set from database warehouse\n"
             + "2)To create data set from text file warehouse" );
      c.add( out );

      // mode
      JLabel m = new JLabel( "Mode" );
      c.add( m );
      // setting mode
      selector.setMode( "Manual" );
      // for mode selection
      names = new String[2];
      names[0] = "Manual";
      names[1] = "Auto";
      cb = new JComboBox( names );
      // adding actionListener to change mode of operation
      cb.addItemListener(
         new ItemListener() {
            public void itemStateChanged( ItemEvent e ) {
               if( e.getStateChange() == ItemEvent.SELECTED ) {
                  selector.setMode( names[cb.getSelectedIndex()] );
               }
            }
         }
      );
      c.add( cb );
      // button to store data to warehouses
      stwh = new JButton( "Store Data" );
      c.add( stwh );
      // button to get host data to warehouses
      gtwh = new JButton( "Host" );
      c.add( gtwh );
      // button to create warehouses
      cwht = new JButton( "Create Tables" );
      c.add( cwht );
      // button to delete warehouses
      dwht = new JButton( "Delete Tables" );
      c.add( dwht );
      // button to create data set once relevant user input is collected
      cds = new JButton( "Data Set" );
      c.add( cds );

      setSize( 650, 450 );
      setVisible( true );
   }

   /**
     * Method actionPerformed creates user inerfaces to create data set
     *
     * @param       et      et is an ActionEvent object
     */

   public void actionPerformed( ActionEvent et ) {
      // if selection structure to check user id and password for database access
      if( et.getSource() == t1
          || et.getSource() == u1
             || et.getSource() == prot1
                || et.getSource() == pa1 ) {
         // for appropriate display
         index = 20;
         // the singleton object which will store user id and password for database access
         DBLogin login = DBLogin.Factory();
         // getting user id
         String uid = login.getUserID();
         // getting password
         String pwd = login.getPassword();
         // if selection structure to get user id and password for database access from the user
         if( uid == null || pwd == null) {
           // will create a second user interface to get user id and password
           // for database access from the user
            DBLoginInfo lifo = new DBLoginInfo( out, index );
         } else {
            out.setText("Please press \"Data Set\" button now");
         }
      }
      UserInput ui = null;
      // if-else structure to create a third user interface
      // to get the relevant user input for the data set
      // and to create the data set
      if( et.getSource() == t1 ) {
         // for selection of time based data set from database warehouse
         index = 1;
         // user interface for time based data set
         ui = new TimeBasedDSUserInput( out );
      } else if( et.getSource() == u1 ) {
         // for selection of user based data set from database warehouse
         index = 2;
         // user interface for user based data set
         ui = new UserBasedDSUserInput( out );
      } else if( et.getSource() == prot1 ) {
         // for selection of protocol based data set from database warehouse
         index = 3;
         // user interface for protocol based data set
         ui = new ProtocolBasedDSUserInput( out );
      } else if( et.getSource() == pa1 ) {
         // for selection of path based data set from database warehouse
         index = 4;
         // user interface for path based data set
         ui = new PathBasedDSUserInput( out );
      } else if( et.getSource() == tt1 ) {
         // for selection of time based data set from text file warehouse
         index = 11;
         // user interface for time based data set
         ui = new TimeBasedDSUserInput( out );
      } else if( et.getSource() == ut1 ) {
         // for selection of user based data set from text file warehouse
         index = 12;
         // user interface for user based data set
         ui = new UserBasedDSUserInput( out );
      } else if( et.getSource() == prott1 ) {
         // for selection of protocol based data set from text file warehouse
         index = 13;
         // user interface for protocol based data set
         ui = new ProtocolBasedDSUserInput( out );
      } else if( et.getSource() == pat1 ) {
         // for selection of path based data set from text file warehouse
         index = 14;
         // user interface for path based data set
         ui = new PathBasedDSUserInput( out );
      } else if( et.getSource() == cds ) {
         // data set file name
         String fileName = getFileName();
         // data set length
         int len = getDSLength();

         // will generate data set
         DataSetGenerator dsg = null;
         // if-else selection structure for the appropriate DataSetGenerator
         // based on the manu selected by the user
         if( index == 1 ) {
            // for time based data set from database warehouse
            dsg = new DataSetGenerator( fileName, len, 1, out );
         } else if( index == 2 ) {
            // for user based data set from database warehouse
            dsg = new DataSetGenerator( fileName, len, 2, out );
         } else if( index == 3 ) {
            // for protocol based data set from database warehouse
            dsg = new DataSetGenerator( fileName, len, 3, out );
         } else if( index == 4 ) {
            // for path based data set from database warehouse
            dsg = new DataSetGenerator( fileName, len, 4, out );
         } else if( index == 11 ) {
            // for time based data set from text file warehouse
            dsg = new DataSetGenerator( fileName, len, 11, out );
         } else if( index == 12 ) {
            // for user based data set from text file warehouse
            dsg = new DataSetGenerator( fileName, len, 12, out );
         } else if( index == 13 ) {
            // for protocol based data set from text file warehouse
            dsg = new DataSetGenerator( fileName, len, 13, out );
         } else if( index == 14 ) {
            // for path based data set from text file warehouse
            dsg = new DataSetGenerator( fileName, len, 14, out );
         }
         // to create the selected data set
         dsg.operation();
      }
      // 'Data Set' will be added to the actionListener once
      if( !cdsAdded ) {
         // 'Data Set' is added to the actionListener
         cds.addActionListener( this );
         cdsAdded = true;
      }
    }

   /**
     * Method getFileName will get file name for the data set from the user
     *
     * @return                                 file name temp as String
     */

   public String getFileName() {
      String temp = "";
      String path = config.getPath();

      //prompt
      temp = JOptionPane.showInputDialog( null,
                                          "Please enter a file name for the data set.",
                                          "Prompt for data set name",
                                          JOptionPane.PLAIN_MESSAGE );
      // creating an File object
      File f = new File( path + "dataset/" + temp );
      // while loop to get a unique file name
      while( f.exists() ) {
         // prompt
         temp = JOptionPane.showInputDialog( null,
                                             temp + " exists. Please enter another file name for the data set.",
                                             "Prompt for data set name",
                                             JOptionPane.PLAIN_MESSAGE );
         // creating an File object
         f = new File( path + "dataset/" + temp );
      }

      return temp;
    }

    /**
      * Method getDSLength will get length of the data set from the user
      *
      * @return                           file length len as int
      */

   public int getDSLength() {
     int len = 0;
     String temp = "";

     //prompt
     temp = JOptionPane.showInputDialog( null,
                                         "Please enter number of access records \nto be included into the data set",
                                         "Prompt for data set size",
                                          JOptionPane.PLAIN_MESSAGE);
     len = Integer.parseInt( temp );

     return len;
   }

   /**
     * The main program
     *
     * @param       args          command line arguments (ignored)
     */

   public static void main( String[] args ) {
      // for an object of WebAccessLogMonitor
      WebAccessLogMonitor slm = new WebAccessLogMonitor();
      // close the window on exit
      slm.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
   }

   /**
     * This is a private class for handling events to create tables of the warehouse
     *
     */
   private class CreateEventHandler implements ActionListener {

    /**
      * Method actionPerformed creates Table object to build the warehouse
      *
      * @param       et      et is an ActionEvent object
      */

      public void actionPerformed( ActionEvent et ) {
         if( et.getSource() == cidb ) {
            // for appropriate display by DBLoginInfo object
            // and to build data warehouse tables in database
            index = 21;
            // the singleton object which will store user id and password for database access
            DBLogin login = DBLogin.Factory();
            // getting user id
            String uid = login.getUserID();
            // getting password
            String pwd = login.getPassword();
            // if selection structure to get user id and password for database access from the user
            if( uid == null || pwd == null ) {
               // will create a second user interface to get user id and password
               // for database access from the user
               DBLoginInfo lifo = new DBLoginInfo( out, index );
            } else {
               out.setText( "Please press \"Create Tables\" button now");
            }
         } else if( et.getSource() == citf ) {
            // to build data warehouse in text file
            index = 22;
            out.setText( "Please press \"Create Tables\" button now");
         } else if( et.getSource() == cwht ) {
            Table tbl = new Table( index, out );
            // will build data warehouse
            tbl.operation();
         }
         // 'Create Tables' will be added to the actionListener once
         if ( !cwhtAdded ) {
            // 'Create Tables' is added to the actionListener
            cwht.addActionListener( this );
            cwhtAdded = true;
         }
      }
   }

   /**
     * This is a private class for handling events to delete tables of the warehouse
     *
     */
   private class DeleteEventHandler implements ActionListener {

    /**
      * Method actionPerformed creates Table object to delete the warehouse
      *
      * @param       et      et is an ActionEvent object
      */

      public void actionPerformed( ActionEvent et ) {
         if( et.getSource() == dfdb ) {
            // for appropriate display by DBLoginInfo object
            // and to delete data warehouse tables from database
            index = 23;
            // the singleton object which will store user id and password for database access
            DBLogin login = DBLogin.Factory();
            // getting user id
            String uid = login.getUserID();
            // getting password
            String pwd = login.getPassword();
            // if selection structure to get user id and password for database access from the user
            if( uid == null || pwd == null ) {
               // will create a second user interface to get user id and password
               // for database access from the user
               DBLoginInfo lifo = new DBLoginInfo( out, index );
            } else {
               out.setText( "Please press \"Delete Tables\" button now");
            }
         } else if( et.getSource() == dftf ) {
            // to delete data warehouse from text file
            index = 24;
            out.setText( "Please press \"Delete Tables\" button now");
         } else if( et.getSource() == dwht ) {
            Table tbl = new Table( index, out );
            // will delete data warehouse
            tbl.operation();
         }
         // 'Delete Tables' will be added to the actionListener once
         if ( !dwhtAdded ) {
            // 'Delete Tables' is added to the actionListener
            dwht.addActionListener( this );
            dwhtAdded = true;
         }
      }
   }

   /**
     * This is a private class for handling events to store data to the warehouse
     *
     */
   private class StoreEventHandler implements ActionListener {

      /**
        * Method actionPerformed creates DataCollector object to store data to the warehouse
        *
        * @param       et      et is an ActionEvent object
        */

      public void actionPerformed( ActionEvent et ) {
         if( et.getSource() == stdb ) {
            // for appropriate display by DBLoginInfo object
            // and to store data to database warehouse
            index = 25;
            // the singleton object which will store user id and password for database access
            DBLogin login = DBLogin.Factory();
            // getting user id
            String uid = login.getUserID();
            // getting password
            String pwd = login.getPassword();
            // if selection structure to get user id and password for database access from the user
            if( uid == null || pwd == null ) {
               // will create a second user interface to get user id and password
               // for database access from the user
               DBLoginInfo lifo = new DBLoginInfo( out, index );
            } else {
               out.setText("Please press \"Store Data\" button now");
            }
         } else if( et.getSource() == sttf ) {
            // to store data to text file warehouse
            index = 26;
            out.setText( "Please press \"Store Data\" button now" );
         } else if( et.getSource() == stwh ) {
            boolean flag = true;

            // while loop to run the program continuously
            while( flag ) {
               DataCollector dc = new DataCollector( index, out );
               // will store data in the warehouse
               dc.operation();

               // for 'Once' mode
               String mode = selector.getMode();
               if( mode.equals( "Manual" ) ) {
                  // go out of the while loop
                  break;
               }
               // getting wait time before next iteration
               int t = config.getSleepingTime();
               MyThread mt = new MyThread( "SleepingThread", t );
               try {
                  // running a MyTread object in second thread
                  mt.start();
                  // waiting fo the second thread to finish
                  mt.join();
               } catch( InterruptedException exception ) {
                  System.err.println( exception.toString() );
               }catch( Exception e ) {
                  System.err.println( e.toString() );
               }
               // checking if configaration is changed to go out of the while loop
               flag = config.doNextIteration();
            }
         }
         // 'Store Data' will be added to the actionListener once
         if ( !stwhAdded ) {
            // 'Store Data' is added to the actionListener
            stwh.addActionListener( this );
            stwhAdded = true;
         }
      }
   }

   /**
     * This is a private class for handling events to get host name of user
     * and to store it to the warehouse
     *
     */
   private class GetEventHandler implements ActionListener {

      /**
        * Method actionPerformed creates HostNameCollector object to get host name
        * and to store it to the warehouse
        *
        * @param       et      et is an ActionEvent object
        */

      public void actionPerformed( ActionEvent et ) {
         if( et.getSource() == gtdb ) {
            // for appropriate display by DBLoginInfo object
            // and to store data to database warehouse
            index = 27;
            // the singleton object which will store user id and password for database access
            DBLogin login = DBLogin.Factory();
            // getting user id
            String uid = login.getUserID();
            // getting password
            String pwd = login.getPassword();
            // if selection structure to get user id and password for database access from the user
            if( uid == null || pwd == null ) {
               // will create a second user interface to get user id and password
               // for database access from the user
               DBLoginInfo lifo = new DBLoginInfo( out, index );
            } else {
               out.setText("Please press \"Host\" button now");
            }
         } else if( et.getSource() == gttf ) {
            // to store data to text file warehouse
            index = 28;
            out.setText( "Please press \"Host\" button now" );
         } else if( et.getSource() == gtwh ) {
            HostNameCollector hnc = new HostNameCollector( index, out );
            // will get host name and store it in to the warehouse
            hnc.operation();
         }
         // 'Host' will be added to the actionListener once
         if ( !gtwhAdded ) {
            // 'Host' is added to the actionListener
            gtwh.addActionListener( this );
            gtwhAdded = true;
         }
      }
   }
} // WebAccessLogMonitor
